import click

from click_configfile import ConfigFileReader, SectionSchema, matches_section
from hamlet.utils import ConfigParam

from hamlet.backend.engine.exceptions import (
    EngineStoreMissingEngineException,
)

from hamlet.backend.engine import EngineStore


class ConfigSchema(object):
    """Schema for standard configuration."""

    @matches_section("default")
    class CliConfig(SectionSchema):
        """base level configuration of the cli itself"""

        log_level = ConfigParam(
            name="log_level",
            type=click.Choice(
                ["fatal", "error", "warn", "info", "debug", "trace"],
                case_sensitive=False,
            ),
        )
        log_format = ConfigParam(
            name="log_format",
            type=click.Choice(["compact", "full"], case_sensitive=False),
        )

    @matches_section("profile:*")
    class Profile(SectionSchema):
        """Profile-specific configuration schema."""

        root_dir = ConfigParam(
            name="root_dir",
            type=click.Path(
                file_okay=False,
                dir_okay=True,
                writable=True,
                resolve_path=True,
                allow_dash=False,
            ),
        )
        district_type = ConfigParam(name="district_type", type=str)
        tenant = ConfigParam(name="tenant", type=str)
        account = ConfigParam(name="account", type=str)
        product = ConfigParam(name="product", type=str)
        environment = ConfigParam(name="environment", type=str)
        segment = ConfigParam(name="segment", type=str)
        engine = ConfigParam(name="engine", type=str)


class ConfigReader(ConfigFileReader):
    """Reader for standard configuration."""

    config_files = ["config.ini", "config"]
    config_name = "standard"
    config_section_schemas = [ConfigSchema.Profile, ConfigSchema.CliConfig]

    @classmethod
    def select_config_schema_for(cls, section_name):
        section_schema = super(ConfigReader, cls).select_config_schema_for(section_name)
        for v in section_schema.__dict__:
            if isinstance(v, ConfigParam):
                v.ctx = cls
        return section_schema

    @classmethod
    def read_config(cls, config_searchpath):
        cls.config_searchpath = config_searchpath
        return super().read_config()

    def load_config(self, opts, profile=None):
        """Load a configuration file into an options object."""
        config = self.read_config(self.config_searchpath)
        values = config.get("default", {})
        self._load_values_into_opts(opts, values)

        if profile:
            values = config.get(f"profile:{profile}", {})
            self._load_values_into_opts(opts, values)

        return values

    @staticmethod
    def _load_values_into_opts(opts, values):
        for k, v in values.items():
            if v is None:
                continue
            if isinstance(v, str):
                if v.startswith('"') or v.startswith("'"):
                    v = v[1:]
                if v.endswith('"') or v.endswith("'"):
                    v = v[:-1]
                if not v:
                    continue
            else:
                if v is None:
                    continue
            setattr(opts, k, v)


class Options:
    """Options object that holds config for the application."""

    def __init__(self, *args, **kwargs):
        """Initialise a new Options object."""
        super(Options, self).__init__(*args, **kwargs)
        self.opts = {}
        for k, v in kwargs:
            setattr(self, k, v)

        self.default_engine_name = "train"
        self._engine_store = None
        self._engine = None

    def get_config_reader(self):
        """Get the config reader class."""
        return ConfigReader()

    def load_config_file(self, profile=None, searchpath=None):
        """Load the config file."""
        config_cls = self.get_config_reader()
        if searchpath:
            config_cls.config_searchpath = [searchpath]
        return config_cls.load_config(self, profile=profile)

    @property
    def cli_config_dir(self):
        """The cli config dir"""
        return self._get_option("cli_config_dir")

    @cli_config_dir.setter
    def cli_config_dir(self, value):
        """Set the cli config dir"""
        self._set_option("cli_config_dir", value)

    @property
    def hamlet_home_dir(self):
        """The home dir used by hamlet"""
        return self._get_option("hamlet_home_dir")

    @hamlet_home_dir.setter
    def hamlet_home_dir(self, value):
        """set the hamlet home dir"""
        self._set_option("hamlet_home_dir", value)

    @property
    def log_level(self):
        """Get the log_level setting"""
        return self._get_option("log_level")

    @log_level.setter
    def log_level(self, value):
        """Set the log_level setting"""
        self._set_option("log_level", value)

    @property
    def root_dir(self):
        """Get the root dir setting"""
        return self._get_option("root_dir")

    @root_dir.setter
    def root_dir(self, value):
        """Set the root_dir setting"""
        self._set_option("root_dir", value)

    @property
    def district_type(self):
        """Get the district type setting"""
        return self._get_option("district_type")

    @district_type.setter
    def district_type(self, value):
        """Set the district type setting"""
        self._set_option("district_type", value)

    @property
    def tenant(self):
        """Get the tenant setting"""
        return self._get_option("tenant")

    @tenant.setter
    def tenant(self, value):
        """Set the tenant setting"""
        self._set_option("tenant", value)

    @property
    def account(self):
        """Get the account setting"""
        return self._get_option("account")

    @account.setter
    def account(self, value):
        """Set the account setting"""
        self._set_option("account", value)

    @property
    def product(self):
        """Get the product setting"""
        return self._get_option("product")

    @product.setter
    def product(self, value):
        """Set the product setting"""
        self._set_option("product", value)

    @property
    def environment(self):
        """Get the environment setting"""
        return self._get_option("environment")

    @environment.setter
    def environment(self, value):
        """Set the environment setting"""
        self._set_option("environment", value)

    @property
    def segment(self):
        """Get the segment setting"""
        return self._get_option("segment")

    @segment.setter
    def segment(self, value):
        """Set the segment setting"""
        self._set_option("segment", value)

    @property
    def generation_framework(self):
        """Get the generation_framework setting"""
        return self._get_option("generation_framework")

    @generation_framework.setter
    def generation_framework(self, value):
        """Set the generation_framework setting"""
        self._set_option("generation_framework", value)

    @property
    def generation_provider(self):
        """Get the generation_provider setting"""
        return self._get_option("generation_provider")

    @generation_provider.setter
    def generation_provider(self, value):
        """Set the generation_provider setting"""
        self._set_option("generation_provider", value)

    @property
    def generation_input_source(self):
        """Get the generation_input_source setting"""
        return self._get_option("generation_input_source")

    @generation_input_source.setter
    def generation_input_source(self, value):
        """Set the generation_input_source setting"""
        self._set_option("generation_input_source", value)

    def _get_option(self, name, default=None):
        """Get value for an option."""
        value = self.opts.get(name)
        if value is None:
            return default
        return value

    def _set_option(self, name, value, allow_clear=False):
        """Set value for an option."""
        if not allow_clear:
            # Prevent clears if value was set
            try:
                current_value = self._get_option(name)
                if value is None and current_value is not None:
                    return
            except AttributeError:
                pass
        self.opts[name] = value

    @property
    def engine_store(self):
        return self._engine_store

    @property
    def engine(self):
        return self._engine

    def set_engine_store(self, engine_dir, config_search_paths):
        self._engine_store = EngineStore(
            store_dir=engine_dir, config_search_paths=config_search_paths
        )

    def set_engine(self, engine_name, locations):
        if self.engine_store:
            engine_name = (
                engine_name
                or self.engine_store.default_engine
                or self.default_engine_name
            )

            try:
                self.engine_store.load_engines(locations=["installed"])
                self._engine = self.engine_store.get_engine(
                    engine_name, locations=["installed"]
                )

            except EngineStoreMissingEngineException:
                self.engine_store.load_engines(locations=locations)
                self.engine_store.get_engine(engine_name, locations=locations).install()
                self.engine_store.load_engines(locations=["installed"], refresh=True)

                self._engine = self.engine_store.get_engine(
                    engine_name, locations=["installed"]
                )


pass_options = click.make_pass_decorator(Options, ensure=True)



import os
import shutil
from zipfile import ZipFile, BadZipFile, LargeZipFile


def run(SourcePath, DestinationPath, env={}):
    """
    Create a zip of a given path and save to a destination
    If the source is a zip then just copy it
    """

    if os.path.isfile(SourcePath):
        create_zip = False
        try:
            source_zip = ZipFile(SourcePath, "r")
            source_zip.testzip()
        except (BadZipFile, LargeZipFile):
            create_zip = True

        if create_zip:
            with ZipFile(DestinationPath, "w") as zip:
                zip.write(SourcePath, os.path.basename(SourcePath))

        else:
            shutil.copyfile(SourcePath, DestinationPath)

    if os.path.isdir(SourcePath):
        shutil.make_archive(os.path.splitext(DestinationPath)[0], "zip", SourcePath)

    return {
        "Properties": {"source_path": SourcePath, "destination_path": DestinationPath}
    }



import collections
from unittest import mock
from click.testing import CliRunner
from hamlet.command.generate.cmdb import generate_tenant
from tests.unit.command.generate.test_generate_command import run_generate_command_test

OPTIONS = collections.OrderedDict()
OPTIONS["!--tenant-id,tenant_id"] = "test-tenant-id"
OPTIONS["--tenant-name,tenant_name"] = ("test-tenant-name", "test-tenant-id")
OPTIONS["--default-region,default_region"] = ("ap-southwest-1", "ap-southeast-2")
OPTIONS["--audit-log-expiry-days,audit_log_expiry_days"] = (10, 2555)
OPTIONS["--audit-log-offline-days,audit_log_offline_days"] = (11, 90)


@mock.patch("hamlet.command.generate.cmdb.generate_tenant_backend")
def test(generate_tenant_backend):
    run_generate_command_test(
        CliRunner(), generate_tenant, generate_tenant_backend.run, OPTIONS
    )



import collections
from unittest import mock
from click.testing import CliRunner
from hamlet.command.manage.crypto import crypto as manage_crypto
from tests.unit.command.test_option_generation import (
    run_options_test,
    run_validatable_option_test,
)


ALL_VALID_OPTIONS = collections.OrderedDict()
ALL_VALID_OPTIONS["-a,--alias"] = "alias"
ALL_VALID_OPTIONS["-b,--base64-decode"] = [True, False]
ALL_VALID_OPTIONS["-d,--decrypt"] = [True, False]
ALL_VALID_OPTIONS["-e,--encrypt"] = [True, False]
ALL_VALID_OPTIONS["-f,--crypto-file"] = "crypto_file"
ALL_VALID_OPTIONS["-k,--key-id"] = "key_id"
ALL_VALID_OPTIONS["-n,--no-alteration"] = [True, False]
ALL_VALID_OPTIONS["-p,--json-path"] = "path.inside.json.file"
ALL_VALID_OPTIONS["-q,--quiet"] = [True, False]
ALL_VALID_OPTIONS["-t,--crypto-text"] = "ciphertext"
ALL_VALID_OPTIONS["-u,--update"] = [True, False]
ALL_VALID_OPTIONS["-v,--visible"] = [True, False]


@mock.patch("hamlet.command.manage.crypto.manage_crypto_backend")
def test_input_valid(manage_crypto_backend):
    runner = CliRunner()
    with runner.isolated_filesystem():
        open("crypto_file", "w").close()
        run_options_test(
            runner, manage_crypto, ALL_VALID_OPTIONS, manage_crypto_backend.run
        )


@mock.patch("hamlet.command.manage.crypto.manage_crypto_backend")
def test_input_validation(manage_crypto_backend):
    runner = CliRunner()
    with runner.isolated_filesystem():
        open("crypto_file", "w").close()
        run_validatable_option_test(
            runner,
            manage_crypto,
            manage_crypto_backend.run,
            {},
            [("-f", "nonexistingfile", "crypto_file")],
        )



import importlib
import click
from hamlet.backend.contract.tasks.exceptions import (
    TaskConditionFailureException,
    TaskFailureException,
)


def run(contract, silent, engine, env):

    properties = contract["Properties"]

    for stage in contract["Stages"]:

        if not silent:
            click.echo(f"\n[-] Running contract stage {stage['Id']}")

        for step in stage["Steps"]:
            if not silent:
                click.echo(f"      Step: {step['Id']} - Task: {step['Type']}")

            try:
                task = importlib.import_module(
                    f"hamlet.backend.contract.tasks.{step['Type']}"
                )
            except ImportError as e:
                raise TaskFailureException(str(e))

            parameters = step["Parameters"]
            replaced_params = {}

            for k, v in parameters.items():
                replaced_params[k] = v
                if isinstance(v, str):
                    substitutions = v.split("__")
                    for substitution in substitutions:
                        if substitution.startswith("Properties:"):
                            property = substitution.split(":", 1)[1]

                            replacement_value = properties.get(property, "")
                            if isinstance(replacement_value, bool):
                                replaced_params[k] = replacement_value
                            else:
                                replaced_params[k] = replaced_params[k].replace(
                                    f"__Properties:{property}__",
                                    str(properties.get(property, "")),
                                )

            replaced_params["env"] = {**engine.environment, **env}
            try:
                task_result = task.run(**replaced_params)

                try:
                    for k, v in task_result["Properties"].items():
                        properties[f"output:{step['Id']}:{k}"] = v
                except KeyError:
                    pass

            except TaskConditionFailureException as e:
                if step["Status"] == "skip_stage_if_failure":
                    break
                else:
                    raise e

    contract["Properties"] = properties

    return contract



import os
import json
import tempfile
import pytest
from hamlet.backend.test.templates.json_validator_obj_block import JSONValidator
from hamlet.backend.test.templates.json_structure_obj_block import JSONStructure
from hamlet.backend.test.templates.cfn_structure_obj_block import CFNStructure
from hamlet.backend.test.templates.cfn_lint_test_func_block import cfn_lint_test
from hamlet.backend.test.templates.checkov_test_func_block import checkov_test
from hamlet.backend.test.testcase_schema import Testcase
from .conftest import DATA_DIR

CF_TEMPLATES_PATH = os.path.join(DATA_DIR, "cf")
CFNStructure = CFNStructure(JSONValidator)
JSONStructure = JSONStructure(JSONValidator)


def test_checkov_test():
    checkov_test(os.path.join(CF_TEMPLATES_PATH, "secure.json"), "cloudformation")
    with pytest.raises(AssertionError):
        checkov_test(os.path.join(CF_TEMPLATES_PATH, "insecure.json"), "cloudformation")


def test_cfn_lint_test():
    cfn_lint_test(os.path.join(CF_TEMPLATES_PATH, "valid-syntax.json"))
    with pytest.raises(AssertionError):
        cfn_lint_test(os.path.join(CF_TEMPLATES_PATH, "invalid-syntax.json"))


def test_json_structure_object():
    body = {
        "path": {"exists": []},
        "not": {
            "empty": {
                "list": [1, 2],
                "scalar": 1,
                "obj": {"1": "value", "2": "value", "3": "value"},
            }
        },
        "empty": {"list": [], "scalar": None, "obj": {}},
    }

    template = JSONStructure(body)

    template.match("path.exists", [])
    template.len("not.empty.list", 2)
    template.len("not.empty.obj", 3)

    template.exists("path.exists")

    template.not_empty("not.empty.list")
    template.not_empty("not.empty.scalar")
    template.not_empty("not.empty.obj")
    assert not template.errors

    template = JSONStructure(body)
    template.match("path.exists", [1])
    template.match("not.empty.scalar", 10)
    template.match("not.empty.obj", {"1": "value"})
    assert len(template.errors) == 3

    template = JSONStructure(body)
    template.len("not.empty.list", 3)
    template.len("not.empty.obj", 2)
    assert len(template.errors) == 2

    template = JSONStructure(body)
    template.exists("not.path.exists")
    assert len(template.errors) == 1

    template = JSONStructure(body)
    template.not_empty("empty.list")
    template.not_empty("empty.scalar")
    template.not_empty("empty.obj")
    assert len(template.errors) == 3


def test_cfn_structure_object():
    body = {
        "Resources": {"TestResource": {"Type": "TestType"}},
        "Outputs": {"TestOutput": {"Property": "Value"}},
    }

    template = CFNStructure(body)
    template.resource("TestResource", "TestType")
    template.output("TestOutput")
    assert not template.errors
    template = CFNStructure(body)
    template.resource("MissingResource", "TestType")
    assert len(template.errors) == 1
    template = CFNStructure(body)
    template.resource("TestResource", "WrongType")
    assert len(template.errors) == 1
    template = CFNStructure(body)
    template.output("MissingOutput")
    assert len(template.errors) == 1


def test_json_validator_object():
    with tempfile.TemporaryDirectory() as temp_dir:
        # testing from_file classmethod
        template_filename = os.path.join(temp_dir, "template.json")

        with pytest.raises(AssertionError) as einfo:
            JSONValidator.from_file(template_filename)
        assert str(einfo.value) == f"{template_filename} not found"

        open(template_filename, "w").close()
        with pytest.raises(AssertionError) as einfo:
            JSONValidator.from_file(template_filename)
        assert str(einfo.value) == f"{template_filename} is not a valid JSON"

        with open(template_filename, "wt") as f:
            json.dump({"key": "value"}, f)

        # testing assert_structure method that uses collection of validators
        validator = JSONValidator.from_file(template_filename)

        def validator_func():
            raise AssertionError("Text")

        validator._validators.append(validator_func)
        with pytest.raises(AssertionError) as einfo:
            validator.assert_structure()
        assert json.loads(str(einfo.value)) == [
            {"rule": "validator_func", "msg": "Text"}
        ]

        validator = JSONValidator.from_file(template_filename)
        validator.assert_structure()


def test_testcase_schema():

    Testcase().load({"filename": "Test"})

    Testcase().load(
        {
            "filename": "Test",
            "cfn_lint": {},
            "checkov": {"framework": "cloudformation"},
            "json_structure": {
                "match": [{"path": "testpath", "value": ["TestValue"]}],
                "exists": [{"path": "testpath"}],
                "length": [{"path": "testpath", "value": 10}],
                "not_empty": [{"path": "testpath"}],
            },
            "cfn_structure": {
                "resource": [{"id": "resource_id", "type": "resource_type"}],
                "output": [{"id": "output_id"}],
            },
        }
    )



from marshmallow import Schema, fields


class Entities(Schema):
    entityID = fields.String(required=True)
    groupID = fields.String(required=True)
    type = fields.String(required=True)
    entityName = fields.String(required=True)


class Relationships(Schema):
    startEntityID = fields.String(required=True)
    endEntityID = fields.String(required=True)
    direction = fields.String(required=True)


class Groups(Schema):
    groupID = fields.String(required=True)
    parentID = fields.String(required=True)


class Diagram(Schema):
    Metadata = fields.Raw()
    diagramName = fields.String(required=True)
    entities = fields.Nested(Entities(many=True))
    relationships = fields.Nested(Relationships(many=True))
    groups = fields.Nested(Groups(many=True))



import os
from unittest import mock

from hamlet.backend.generate.cmdb import account, tenant, product
from hamlet.backend.create.template.environment import Environment
from hamlet.backend.create.template.set_context import set_context
from .conftest import conf


def test(cmdb, clear_cmdb):
    clear_cmdb()

    with mock.patch.dict(os.environ, {"ROOT_DIR": cmdb.ROOT_DIR}):

        tenant_id = conf["cmdb"]["tenant"]["tenant_id"]
        tenant_name = conf["cmdb"]["tenant"]["tenant_name"]
        account_id = conf["cmdb"]["account"]["account_id"]
        account_name = conf["cmdb"]["account"]["account_name"]
        product_id = conf["cmdb"]["product"]["product_id"]
        product_name = conf["cmdb"]["product"]["product_name"]

        with cmdb() as path:
            tenant.run(**conf["cmdb"]["tenant"], output_dir=path())
        with cmdb("accounts") as path:
            account.run(**conf["cmdb"]["account"], output_dir=path())
        with cmdb() as path:
            product.run(**conf["cmdb"]["product"], output_dir=path())
        with cmdb(product_name, "config") as path:
            e = Environment({"ACCOUNT": account_name})
            set_context(cwd=path(), environment_obj=e)

        with cmdb() as path:
            assert e.AID == account_id
            assert e.ACCOUNT == account_name
            assert e.ACCOUNT_DIR == os.path.join(
                path(), "accounts", account_name, "config"
            )
            assert e.TID == tenant_id
            assert e.TENANT == tenant_name
            assert e.TENANT_DIR == os.path.join(path(), "accounts", tenant_name)
            assert e.PID == product_id
            assert e.PRODUCT == product_name
            assert e.PRODUCT_DIR == os.path.join(path(), product_name, "config")
            print(e)



import click
import functools
import json
import textwrap

from tabulate import tabulate

MAX_TABLE_TEXT_CONTENT_WIDTH = 128


def wrap_text(text):
    if text is None:
        return "None"
    if isinstance(text, int):
        return text
    return "\n".join(textwrap.wrap(text, MAX_TABLE_TEXT_CONTENT_WIDTH))


def table_from_dict(data):
    """
    creates a table of k,v pairs from a list of dicts
    """
    tablerows = []
    for k, v in data.items():
        tablerows.append([wrap_text(k), wrap_text(v)])
    return tabulate(tablerows, headers=["Key", "Value"], tablefmt="github")


def json_or_table_option(tablefunc):
    def decorator(func):
        click.option(
            "--output-format",
            "output_format",
            type=click.Choice(["table", "json"], case_sensitive=False),
            default="table",
            help="Select output format",
        )(func)

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            output_format = kwargs["output_format"].lower()
            del kwargs["output_format"]
            result = func(*args, **kwargs)
            if output_format == "table":
                click.echo(tablefunc(result))
            elif output_format == "json":
                click.echo(json.dumps(result, indent=4))

        return wrapper

    return decorator



import boto3
import datetime

from hamlet.backend.common.utilities import to_bool


def get_tag_set(dbArn, rds_client):
    return [
        tag
        for tag in rds_client.list_tags_for_resource(ResourceName=dbArn)["TagList"]
        if not tag["Key"].startswith("rds:") and not tag["Key"].startswith("aws:")
    ]


def run(
    DbId=None,
    Cluster=None,
    SnapshotName=None,
    IncludeDateSuffix=None,
    Region=None,
    AWSAccessKeyId=None,
    AWSSecretAccessKey=None,
    AWSSessionToken=None,
    env={},
):
    """
    Create a manual snapshot of an RDS instance or cluster
    """

    session = boto3.Session(
        aws_access_key_id=AWSAccessKeyId,
        aws_secret_access_key=AWSSecretAccessKey,
        aws_session_token=AWSSessionToken,
        region_name=Region,
    )
    rds = session.client("rds")

    SnapshotName = (
        f"{SnapshotName}-{datetime.datetime.now().strftime('%Y%m%d-%H%M')}"
        if to_bool(IncludeDateSuffix, True)
        else SnapshotName
    )

    if to_bool(Cluster, False):

        cluster = rds.describe_db_clusters(DBClusterIdentifier=DbId)

        snapshot_response = rds.create_db_cluster_snapshot(
            DBClusterSnapshotIdentifier=SnapshotName,
            DBClusterIdentifier=DbId,
            Tags=get_tag_set(cluster["DBClusters"][0]["DBClusterArn"], rds),
        )

        waiter = rds.get_waiter("db_cluster_snapshot_available")
        waiter.wait(
            DBClusterSnapshotIdentifier=SnapshotName,
            DBClusterIdentifier=DbId,
            SnapshotType="manual",
        )
        snapshot_arn = snapshot_response["DBClusterSnapshot"]["DBClusterSnapshotArn"]

    else:

        instance = rds.describe_db_instances(DBInstanceIdentifier=DbId)

        snapshot_response = rds.create_db_snapshot(
            DBSnapshotIdentifier=SnapshotName,
            DBInstanceIdentifier=DbId,
            Tags=get_tag_set(instance["DBInstances"][0]["DBInstanceArn"], rds),
        )

        waiter = rds.get_waiter("db_snapshot_completed")
        waiter.wait(
            DBSnapshotIdentifier=SnapshotName,
            DBInstanceIdentifier=DbId,
            SnapshotType="manual",
        )

        snapshot_arn = snapshot_response["DBSnapshot"]["DBSnapshotArn"]

    return {"Properties": {"SnapshotArn": snapshot_arn}}



from httpx import Client, Timeout


class HTTPClient(Client):
    """
    A standard httpx client used in the hamlet cli
    """

    def __init__(
        self,
        *args,
        timeout=Timeout(timeout=5.0, read=120.0),
        follow_redirects: bool = True,
        trust_env: bool = True,
        **kwargs
    ):
        super().__init__(
            *args,
            timeout=timeout,
            follow_redirects=follow_redirects,
            trust_env=trust_env,
            **kwargs
        )



import pytest
from hamlet.backend.create.template import utils


def test_semver():
    assert utils.semver_valid("v1.0.0") == ("1", "0", "0", None, None)
    assert utils.semver_valid("v2.0.1") == ("2", "0", "1", None, None)
    assert utils.semver_valid("2.0.1-a+b") == ("2", "0", "1", "a", "b")
    assert utils.semver_valid("2.0.1+b") == ("2", "0", "1", None, "b")
    assert utils.semver_valid("2.0.1-a") == ("2", "0", "1", "a", None)
    with pytest.raises(ValueError):
        utils.semver_valid("1.0")
    assert utils.semver_compare("1.0.0", "1.0.1") < 0
    assert utils.semver_compare("1.0.1", "1.0.0") > 0
    assert utils.semver_compare("1.0.0", "1.0.0") == 0

    assert utils.semver_clean("v1.2.3") == "1.2.3"
    assert utils.semver_clean("v1.X.x") == "1.0.0"

    assert utils.semver_satisfies("1.0.0", ["<=1.0.0"])
    assert utils.semver_satisfies("1.0.0", [">=1.0.0"])
    assert utils.semver_satisfies("1.0.0", ["<=2.0.0"])
    assert utils.semver_satisfies("1.0.0", [">=0.0.1"])
    assert utils.semver_satisfies("1.0.0", ["=1.0.0"])
    assert utils.semver_satisfies("1.0.0", ["<2.0.0"])
    assert utils.semver_satisfies("1.0.0", [">0.0.1"])

    assert not utils.semver_satisfies("2.0.0", ["<=1.0.0"])
    assert not utils.semver_satisfies("0.0.1", [">=1.0.0"])
    assert not utils.semver_satisfies("1.0.1", ["=1.0.0"])
    assert not utils.semver_satisfies("3.0.0", ["<2.0.0"])
    assert not utils.semver_satisfies("1.0.0", [">2.0.0"])



import typing

from hamlet.backend.engine.engine_loader import EngineLoader

from hamlet.backend.engine.engine import Engine, ShimEngine
from hamlet.backend.engine.engine_part import (
    CoreEnginePart,
    AWSEnginePluginPart,
    AzureEnginePluginPart,
    CMDBEnginePluginPart,
    BashExecutorEnginePart,
    WrapperEnginePart,
    BundledWrapperEnginePart,
)
from hamlet.backend.engine.engine_source import ShimPathEngineSource


class ShimEngineLoader(EngineLoader):
    """
    Seeds an engine used to create a known engine directory that can be used to establish
    a standard location for hamlet scripts called outside of the cli
    """

    def load(self) -> typing.Iterable[Engine]:
        engine_source = [
            ShimPathEngineSource(name="shim", description="shim based root dir source")
        ]

        engine_parts = [
            CoreEnginePart(source_path="engine-core", source_name="shim"),
            BashExecutorEnginePart(source_path="executor-bash", source_name="shim"),
            AWSEnginePluginPart(source_path="engine-plugin-aws", source_name="shim"),
            AzureEnginePluginPart(
                source_path="engine-plugin-azure", source_name="shim"
            ),
            CMDBEnginePluginPart(source_path="engine-plugin-cmdb", source_name="shim"),
            WrapperEnginePart(source_path="engine-wrapper", source_name="shim"),
        ]

        engine = ShimEngine(
            name="shim",
            description="The engine used to provide shim based access to a fixed diretory",
        )
        engine.parts = engine_parts
        engine.sources = engine_source

        yield engine


class BundledShimEngineLoader(EngineLoader):
    """
    Seeds an engine used to create a known engine directory that can be used to establish
    a standard location for hamlet scripts called outside of the cli
    """

    def load(self) -> typing.Iterable[Engine]:
        engine_source = [
            ShimPathEngineSource(name="shim", description="shim based root dir source")
        ]

        engine_parts = [
            CoreEnginePart(source_path="engine-core", source_name="shim"),
            BashExecutorEnginePart(source_path="executor-bash", source_name="shim"),
            AWSEnginePluginPart(source_path="engine-plugin-aws", source_name="shim"),
            AzureEnginePluginPart(
                source_path="engine-plugin-azure", source_name="shim"
            ),
            CMDBEnginePluginPart(source_path="engine-plugin-cmdb", source_name="shim"),
            BundledWrapperEnginePart(source_path="engine-wrapper", source_name="shim"),
        ]

        engine = ShimEngine(
            name="bundled_shim",
            description="The engine used to provide shim based access to a fixed diretory",
        )
        engine.parts = engine_parts
        engine.sources = engine_source

        yield engine



import os
import json
import collections

from unittest import mock
from click.testing import CliRunner

from hamlet.command.task import describe_runbook
from hamlet.command.common.config import Options
from tests.unit.command.test_option_generation import (
    run_options_test,
)


runbookinfo_mock_output = {
    "RunBooks": [
        {
            "Name": "RunBook1",
            "Description": "RunBookDescription1",
            "Engine": "RunBookEngine1",
        },
        {
            "Name": "RunBook2",
            "Description": "RunBookDescription2",
            "Engine": "RunBookEngine2",
        },
    ],
}


def template_backend_run_mock(data):
    def run(
        output_filename="runbookinfo-config.json", output_dir=None, *args, **kwargs
    ):
        os.makedirs(output_dir, exist_ok=True)
        filename = os.path.join(output_dir, output_filename)
        with open(filename, "wt+") as f:
            json.dump(data, f)

    return run


def mock_backend(data=None):
    def decorator(func):
        @mock.patch("hamlet.backend.query.template")
        def wrapper(blueprint_mock, *args, **kwargs):
            blueprint_mock.run.side_effect = template_backend_run_mock(data)
            return func(blueprint_mock, *args, **kwargs)

        return wrapper

    return decorator


DECSCRIBE_RUNBOOK_VALID_OPTIONS = collections.OrderedDict()
DECSCRIBE_RUNBOOK_VALID_OPTIONS["!-n,--name"] = "RunBook1"
DECSCRIBE_RUNBOOK_VALID_OPTIONS["-q,--query"] = "[]"


@mock_backend(runbookinfo_mock_output)
def test_describe_runbook_input_valid(
    blueprint_mock,
):
    run_options_test(
        CliRunner(),
        describe_runbook,
        DECSCRIBE_RUNBOOK_VALID_OPTIONS,
        blueprint_mock.run,
    )


@mock_backend(runbookinfo_mock_output)
def test_describe_runbook(blueprint_mock):
    obj = Options()

    cli = CliRunner()
    result = cli.invoke(
        describe_runbook,
        ["--name", "RunBook1"],
        obj=obj,
    )
    print(result.output)
    assert result.exit_code == 0
    result = json.loads(result.output)
    assert {
        "Name": "RunBook1",
        "Description": "RunBookDescription1",
        "Engine": "RunBookEngine1",
    } == result



import boto3
from simple_term_menu import TerminalMenu
import json


def run(
    Tags=None,
    VpcId=None,
    Region=None,
    AWSAccessKeyId=None,
    AWSSecretAccessKey=None,
    AWSSessionToken=None,
    env={},
):
    """
    Lists the available ec2 instances using a vpc or tag based filter
    Users select one and the returned result is the ec2 instance id
    """
    session = boto3.Session(
        aws_access_key_id=AWSAccessKeyId,
        aws_secret_access_key=AWSSecretAccessKey,
        aws_session_token=AWSSessionToken,
        region_name=Region,
    )
    ec2 = session.resource("ec2")

    filters = []

    if VpcId is not None:
        filters.append({"Name": "vpc-id", "Values": [VpcId]})

    if Tags is not None:
        for k, v in Tags.items():
            filters.append({"Name": f"tag:{k}", "Values": [v]})

    ec2_instances = ec2.instances.filter(Filters=filters)
    instance_ids = [instance.id for instance in ec2_instances]

    def get_instance_preview(instance_id):
        instance = ec2.Instance(instance_id)
        task_summary = {
            "Name": [tag["Value"] for tag in instance.tags if tag["Key"] == "Name"][0],
            "State": instance.state["Name"],
            "PrivateIp": instance.private_ip_address,
            "Type": instance.instance_type,
            "LaunchTime": str(instance.launch_time),
        }

        return json.dumps(task_summary, indent=2)

    menu = TerminalMenu(
        instance_ids,
        title="EC2 Instances",
        preview_command=get_instance_preview,
        preview_title="details",
        preview_size=0.75,
    )

    print("\n")
    index = menu.show()
    instance_id = instance_ids[index]

    print(f"Instance Id: {instance_id}")
    print("\n")

    return {"Properties": {"result": instance_id}}



import hmac
import hashlib
import base64


# These values are required to calculate the signature. Do not change them.
DATE = "11111111"
SERVICE = "ses"
MESSAGE = "SendRawEmail"
TERMINAL = "aws4_request"
VERSION = 0x04


def run(
    SESRegion,
    AWSSecretAccessKey=None,
    env={},
):
    """
    Generate a Sig4 based SMTP password from a provided secret access key
    Used with SES to send emails via SMTP
    """

    def sign(key, msg):
        return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

    signature = sign(("AWS4" + AWSSecretAccessKey).encode("utf-8"), DATE)
    signature = sign(signature, SESRegion)
    signature = sign(signature, SERVICE)
    signature = sign(signature, TERMINAL)
    signature = sign(signature, MESSAGE)
    signature_and_version = bytes([VERSION]) + signature
    smtp_password = base64.b64encode(signature_and_version)

    return {
        "Properties": {
            "smtp_password": smtp_password.decode("utf-8"),
        }
    }



from hamlet.backend.common import runner


def run(
    registry_scope=None,
    dockerfile=None,
    docker_context=None,
    image_formats=None,
    code_commit=None,
    image_paths=None,
    docker_image=None,
    deployment_unit=None,
    log_level=None,
    engine=None,
    _is_cli=False,
    env={},
):

    opts = {
        "-c": registry_scope,
        "-d": dockerfile,
        "-e": docker_context,
        "-f": image_formats,
        "-g": code_commit,
        "-i": image_paths,
        "-t": docker_image,
        "-u": deployment_unit,
    }

    env = {"AUTOMATION_LOG_LEVEL": log_level, **env}
    runner.run(
        "manageImages.sh",
        args=[],
        options=opts,
        env=env,
        engine=engine,
        _is_cli=_is_cli,
        script_base_path_env="AUTOMATION_DIR",
    )



from hamlet.command import root as cli
from .expo_app_publish import expo_app_publish as expo_app_publish_cmd
from .task import task as task_cmd
from .lambda_func import lambda_func as lambda_func_cmd
from .pipeline import pipeline as pipeline_cmd
from .sentry_release import sentry_release as sentry_release_cmd


@cli.group("run", context_settings=dict(max_content_width=240))
def group():
    """
    Runs stuff
    """


group.add_command(expo_app_publish_cmd)
group.add_command(task_cmd)
group.add_command(lambda_func_cmd)
group.add_command(pipeline_cmd)
group.add_command(sentry_release_cmd)



import click

from tabulate import tabulate

from hamlet.command.common.display import json_or_table_option, wrap_text
from hamlet.command.common import config, exceptions

from .common import query_occurrences_state

LIST_OCCURRENCES_QUERY = (
    "Occurrences[]"
    ".{"
    "TierId:Core.Tier.Id,"
    "ComponentId:Core.Component.RawId,"
    "Name:Core.TypedRawName,"
    "Type:Core.Type"
    "}"
)


def list_occurrences_table(data):
    tablerows = []
    for row in data:
        tablerows.append(
            [
                wrap_text(row["TierId"]),
                wrap_text(row["ComponentId"]),
                wrap_text(row["Name"]),
                wrap_text(row["Type"]),
            ]
        )
    return tabulate(
        tablerows, headers=["TierId", "ComponentId", "Name", "Type"], tablefmt="github"
    )


@click.command(
    "list-occurrences", short_help="", context_settings=dict(max_content_width=240)
)
@click.option(
    "-q",
    "--query",
    help="A JMESPath query to apply to the results",
)
@json_or_table_option(list_occurrences_table)
@exceptions.backend_handler()
@config.pass_options
def list_occurrences(options, query):
    """
    List available occurrences
    """
    return query_occurrences_state(
        options=options,
        engine=options.engine,
        query=LIST_OCCURRENCES_QUERY,
        sub_query_text=query,
    )



import pytest
from . import step_1_create_cmdb
from . import step_2_create_account_templates
from . import step_3_create_segment_templates


@pytest.mark.slow
def test_tenant_and_product_setup(engine, cmdb, clear_cmdb):
    step_1_create_cmdb.run(cmdb, clear_cmdb)
    step_2_create_account_templates.run(cmdb, engine)
    step_3_create_segment_templates.run(cmdb, engine)



import typing
import semver

from hamlet.backend.engine.engine_loader import EngineLoader

from hamlet.backend.engine.engine import Engine
from hamlet.backend.engine.engine_part import (
    CoreEnginePart,
    AWSEnginePluginPart,
    AzureEnginePluginPart,
    BashExecutorEnginePart,
    WrapperEnginePart,
    BundledWrapperEnginePart,
)
from hamlet.backend.engine.engine_source import ContainerEngineSource
from hamlet.backend.container_registry import ContainerRepository


class LatestTrainEngineLoader(EngineLoader):
    """
    Provides the latest Train release
    """

    def load(self) -> typing.Iterable[Engine]:

        engine_sources = [
            ContainerEngineSource(
                name="hamlet-engine-base",
                description="hamlet official engine source",
                registry_url="https://ghcr.io",
                repository="hamlet-io/hamlet-engine-base",
                tag="latest",
            ),
        ]

        engine_parts = [
            BundledWrapperEnginePart(
                source_path="engine-core/image/", source_name="hamlet-engine-base"
            ),
            CoreEnginePart(source_path="engine/", source_name="hamlet-engine-base"),
            BashExecutorEnginePart(
                source_path="executor-bash/", source_name="hamlet-engine-base"
            ),
            AWSEnginePluginPart(
                source_path="engine-plugin-aws/", source_name="hamlet-engine-base"
            ),
            AzureEnginePluginPart(
                source_path="engine-plugin-azure/", source_name="hamlet-engine-base"
            ),
        ]

        engine = Engine(
            name="train", description="The latest release of the official hamlet engine"
        )
        engine.parts = engine_parts
        engine.sources = engine_sources

        self._engines = [engine]

        yield engine


class TrainEngineLoader(EngineLoader):
    """
    Provides all of the tram releases which have been created
    These are considered the Train Release Candidates and can be used for managing releases
    """

    def load(self) -> typing.Iterable[Engine]:

        container_repo = ContainerRepository(
            registry_url="https://ghcr.io", repository="hamlet-io/hamlet-engine-base"
        )

        for tag in container_repo.tags:

            try:
                version = semver.VersionInfo.parse(tag)
            except ValueError:
                continue

            if not version.prerelease:

                engine_sources = [
                    ContainerEngineSource(
                        name="hamlet-engine-base",
                        description="hamlet official engine source",
                        registry_url="https://ghcr.io",
                        repository="hamlet-io/hamlet-engine-base",
                        tag=tag,
                    ),
                ]

                engine_parts = [
                    CoreEnginePart(
                        source_path="engine/", source_name="hamlet-engine-base"
                    ),
                    BashExecutorEnginePart(
                        source_path="executor-bash/", source_name="hamlet-engine-base"
                    ),
                    AWSEnginePluginPart(
                        source_path="engine-plugin-aws/",
                        source_name="hamlet-engine-base",
                    ),
                    AzureEnginePluginPart(
                        source_path="engine-plugin-azure/",
                        source_name="hamlet-engine-base",
                    ),
                ]

                if version.match(">=8.5.0"):
                    engine_parts.append(
                        BundledWrapperEnginePart(
                            source_path="engine-core/image/",
                            source_name="hamlet-engine-base",
                        )
                    )
                else:
                    engine_parts.append(
                        WrapperEnginePart(
                            source_path="engine-core/", source_name="hamlet-engine-base"
                        )
                    )

                engine = Engine(
                    name=tag,
                    description="Stable release of the official hamlet engine",
                )
                engine.parts = engine_parts
                engine.sources = engine_sources

                yield engine



import click
from tabulate import tabulate


def confirm(kwargs, **confirm_kwargs):
    confirm_kwargs["text"] = confirm_kwargs.get("text") or "Is everything correct?"
    lines = []
    i = 1
    for key, value in kwargs.items():
        if value is None:
            continue
        name = key.replace("_", " ")
        lines.append([i, name, value])
        i += 1
    click.echo(tabulate(lines, ["№", "parameter", "value"], tablefmt="psql"))
    return click.confirm(**confirm_kwargs)



import click
import json

from hamlet.command.common import config, exceptions

from .common import query_occurrences_state


@click.command(
    "query-occurrences", short_help="", context_settings=dict(max_content_width=240)
)
@click.option(
    "-q",
    "--query",
    required=True,
    help="A JMESPath query to apply to the results",
)
@exceptions.backend_handler()
@config.pass_options
def query_occurrences(options, query):
    """
    Run a query over all occurrences
    """
    result = query_occurrences_state(
        options=options, engine=options.engine, query=query
    )
    click.echo(json.dumps(result, indent=4))



import collections
from unittest import mock
from click.testing import CliRunner
from hamlet.command.manage.stack import stack as manage_stack
from tests.unit.command.test_option_generation import (
    run_options_test,
    run_validatable_option_test,
)


ALL_VALID_OPTIONS = collections.OrderedDict()
ALL_VALID_OPTIONS["!-u,--deployment-unit"] = "unit"
ALL_VALID_OPTIONS["!-l,--deployment-group"] = "group"
ALL_VALID_OPTIONS["-d,--delete"] = [True, False]
ALL_VALID_OPTIONS["-i,--stack-initiate"] = [True, False]
ALL_VALID_OPTIONS["-m,--stack-monitor"] = [True, False]
ALL_VALID_OPTIONS["-r,--region"] = "region"
ALL_VALID_OPTIONS["-w,--stack-wait"] = 10
ALL_VALID_OPTIONS["-z,--deployment-unit-subset"] = "subset"
ALL_VALID_OPTIONS["-y,--dryrun"] = [True, False]


@mock.patch("hamlet.command.manage.stack.manage_stack_backend")
def test_input_valid(manage_stack_backend):
    run_options_test(
        CliRunner(), manage_stack, ALL_VALID_OPTIONS, manage_stack_backend.run
    )


@mock.patch("hamlet.command.manage.stack.manage_stack_backend")
def test_input_validation(manage_stack_backend):
    runner = CliRunner()
    run_validatable_option_test(
        runner,
        manage_stack,
        manage_stack_backend.run,
        {"-u": "unit", "-l": "group"},
        [("-w", "not_an_int", 10)],
    )



import os
import subprocess
import shutil

from .exceptions import BackendException


def __cli_params_to_script_call(
    script_path, script_name, args=None, options=None, extra_script=None
):
    args = list(arg for arg in (args if args is not None else []) if arg is not None)
    options_list = []
    for key, value in options.items():
        if value is not None:
            if isinstance(value, bool):
                if value:
                    options_list.append(str(key))
            elif isinstance(value, tuple):
                for instance in value:
                    options_list.append(str(key))
                    options_list.append(str(f"'{instance}'"))
            else:
                options_list.append(str(key))
                options_list.append(str(f"'{value}'"))
    script_fullpath = os.path.join(script_path, script_name)
    if extra_script is not None:
        return " ".join([".", script_fullpath] + options_list + args + [extra_script])
    return " ".join([script_fullpath] + options_list + args)


def __env_params_to_envvars(env=None):
    cmd_env = {}
    for key, value in env.items():
        if value is not None:
            if isinstance(value, tuple):
                cmd_env[key.upper()] = ",".join(value)
            elif isinstance(value, bool):
                cmd_env[key.upper()] = str(value).lower()
            else:
                cmd_env[key.upper()] = str(value)
    return cmd_env


def run(
    script_name,
    args,
    options,
    env,
    engine,
    _is_cli,
    script_base_path_env="GENERATION_DIR",
    extra_script=None,
):
    env_overrides = {
        **os.environ,
        **__env_params_to_envvars(env),
    }

    if engine:
        env_overrides = {**env_overrides, **engine.environment}

    try:
        os.path.isdir(env_overrides[script_base_path_env])
    except TypeError:
        raise BackendException(
            f"Could not find script base path using env {script_base_path_env}: {env_overrides[script_base_path_env]}"
        )

    if shutil.which("bash") is None:
        raise BackendException("Could not find bash installation")

    try:
        script_call_line = __cli_params_to_script_call(
            env_overrides[script_base_path_env],
            script_name,
            args=args,
            options=options,
            extra_script=extra_script,
        )
        process = subprocess.Popen(
            [shutil.which("bash"), "-c", script_call_line],
            stdout=None if _is_cli else subprocess.PIPE,
            stderr=None if _is_cli else subprocess.PIPE,
            env=env_overrides,
            encoding="utf-8",
            bufsize=1,
        )

        stdout, stderr = process.communicate()
        if not _is_cli and process.returncode != 0:
            exception_message = "\n".join(
                [
                    f"script: {script_call_line}",
                    "",
                    "   stdout",
                    "".join((["#"] * 30)),
                    "",
                    stdout,
                    "   stderr",
                    "".join((["#"] * 30)),
                    "",
                    stderr,
                ]
            )
            raise BackendException(exception_message)
        if _is_cli and process.returncode != 0:
            raise BackendException(f"{script_name} failed to run")

    finally:
        try:
            process.kill()
        except ProcessLookupError:
            pass
        except UnboundLocalError:
            pass



import click
from hamlet.backend.run import sentry_release as run_sentry_release_backend
from hamlet.command.common import exceptions
from hamlet.command.common.config import pass_options


@click.command(
    "sentry-release",
    short_help="",
    context_settings=dict(max_content_width=240),
)
@click.option(
    "-u",
    "--deployment-unit",
    required=True,
    help="The deployment unit of the moobile app",
)
@click.option(
    "-g",
    "--deployment-group",
    default="application",
    help="The deployment group of the moobile app",
)
@click.option(
    "-m", "--sentry-source-map-s3-url", required=True, help="s3 link to sourcemap files"
)
@click.option(
    "-a",
    "--app-type",
    type=click.Choice(["", "react-native"]),
    help="The application framework being used",
)
@click.option(
    "-d",
    "--dist",
    help="A distribution Identifier for the file",
)
@click.option("-p", "--sentry-url-prefix", help="prefix for sourcemap files")
@click.option(
    "-r",
    "--sentry-release-name",
    help="release name",
    required=True,
)
@exceptions.backend_handler()
@pass_options
def sentry_release(options, **kwargs):
    """
    Upload sourcemap files to sentry for a specific release
    """

    args = {**options.opts, **kwargs}
    run_sentry_release_backend.run(**args, engine=options.engine, _is_cli=True)



import click
from hamlet.backend.run import pipeline as run_pipeline_backend
from hamlet.command.common import exceptions
from hamlet.command.common.config import pass_options


@click.command(
    "pipeline",
    short_help="Run an AWS Data pipeline",
    context_settings=dict(max_content_width=240),
    deprecated=True,
)
@click.option(
    "-i",
    "--component",
    help="name of the component in the solution where the task is defined",
    required=True,
)
@click.option(
    "-t",
    "--tier",
    help="name of the tier in the solution where the task is defined",
    required=True,
)
@click.option("-x", "--instance", help="instance of the task to be run")
@click.option("-y", "--version", help="version of the task to be run")
@click.option(
    "-s",
    "--pipeline-status-only",
    help="check the running status of a pipelie",
    is_flag=True,
)
@click.option(
    "-c",
    "--pipeline-allow-concurrent",
    help="activate the pipeline if another one is running",
    is_flag=True,
)
@exceptions.backend_handler()
@pass_options
def pipeline(options, **kwargs):
    """
    Run an AWS Data pipeline

    \b
    NOTES:
    1. This will activate the pipeline and leave it running
    2. Pipelines take a long time so it is better to provide status via other means
    """

    args = {**options.opts, **kwargs}

    run_pipeline_backend.run(**args, engine=options.engine, _is_cli=True)



import click
from hamlet.backend.manage import deployment as manage_deployment_backend
from hamlet.command.common.config import pass_options
from hamlet.command.common import exceptions


@click.command(
    "deployment",
    short_help="Manage an Azure Resource Manager (ARM) deployment",
    context_settings=dict(max_content_width=240),
    deprecated=True,
)
@click.option("-d", "--delete", help="delete the deployment", is_flag=True)
@click.option(
    "-g",
    "--resource-group",
    help="target resource group for the deployment.",
)
@click.option(
    "-i",
    "--deployment-initiate",
    help="initiate but do not monitor the deployment operation(disable monitoring)",
    is_flag=True,
)
@click.option(
    "-l",
    "--deployment-group",
    help="the deployment group the unit belongs to",
    required=True,
)
@click.option(
    "-m",
    "--deployment-monitor",
    help="monitor but do not initiate the deployment operation(disable initiation)",
    is_flag=True,
)
@click.option("-r", "--region", help="Azure location/region code for this deployment")
@click.option(
    "-s",
    "--deployment-scope",
    type=click.Choice(["subscription", "resourceGroup"], case_sensitive=False),
    help="deployment scope",
    default="resourceGroup",
    show_default=True,
)
@click.option(
    "-u",
    "--deployment-unit",
    required=True,
    help="deployment unit used to determine the deployment template",
)
@click.option(
    "-w",
    "--deployment-wait",
    type=click.INT,
    help="interval between checking the progress of a deployment operation",
    default=30,
    show_default=True,
)
@click.option(
    "-z", "--deployment-unit-subset", help="subset of the deployment unit required"
)
@exceptions.backend_handler()
@pass_options
def deployment(options, **kwargs):
    """
    Manage an Azure Resource Manager (ARM) deployment
    """

    args = {**options.opts, **kwargs}

    manage_deployment_backend.run(**args, engine=options.engine, _is_cli=True)



import boto3


def run(
    BucketName,
    Object,
    LocalPath,
    Region=None,
    AWSAccessKeyId=None,
    AWSSecretAccessKey=None,
    AWSSessionToken=None,
    env={},
):
    """
    Upload an object from a local directory to an S3 Bucket
    """

    session = boto3.Session(
        aws_access_key_id=AWSAccessKeyId,
        aws_secret_access_key=AWSSecretAccessKey,
        aws_session_token=AWSSessionToken,
        region_name=Region,
    )

    s3 = session.resource("s3")
    s3.Object(BucketName, Object).upload_file(LocalPath)

    return {
        "Properties": {
            "s3_path": f"s3://{BucketName}/{Object}",
            "local_path": LocalPath,
        }
    }



import functools
import click
from click._compat import get_text_stderr

from httpx import HTTPError
from click.exceptions import ClickException

from hamlet.backend.common.exceptions import BackendException


class CommandError(ClickException):
    """An exception occurred during processing"""

    def show(self, file=None):
        if file is None:
            file = get_text_stderr()
        click.echo(
            click.style("[!] Error Running Command", fg="red", bold=True), err=True
        )
        click.echo("")
        click.echo(self.format_message(), file=file, err=True)

    def format_message(self):
        return click.style(str(self.message))


def backend_handler():
    """Handles backend errors with a formatted click exception message"""

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)

            except HTTPError as e:
                msg = f"HTTP error | url: {e.request.url} | msg: {str(e)}"
                raise CommandError(msg) from e

            except BackendException as e:
                raise CommandError(str(e)) from e

        return wrapper

    return decorator



from hamlet.backend.automation_tasks.base import AutomationRunner
from hamlet.backend.automation import (
    properties_file,
    set_automation_context,
    construct_tree,
    manage_build_references,
)


class UpdateBuildAutomationRunner(AutomationRunner):
    def __init__(
        self,
        deployment_unit=None,
        build_reference=None,
        code_tag=None,
        image_format=None,
        registry_scope=None,
        engine=None,
        **kwargs
    ):
        _context_env = {
            **kwargs,
        }

        super().__init__(engine=engine, **_context_env)

        self._script_list = [
            {
                "func": construct_tree.run,
                "args": {
                    "use_existing_tree": True,
                    "_is_cli": True,
                },
            },
            {
                "func": properties_file.get_automation_properties,
                "args": {**self._context_env},
            },
            {"func": set_automation_context.run, "args": {"_is_cli": True}},
            {
                "func": manage_build_references.run,
                "args": {
                    "_is_cli": True,
                    "verify": "latest",
                    "code_commits": build_reference,
                    "deployment_units": deployment_unit,
                    "image_formats": image_format,
                    "code_tags": code_tag,
                    "registry_scopes": registry_scope,
                },
            },
            {
                "func": manage_build_references.run,
                "args": {
                    "_is_cli": True,
                    "update": True,
                    "code_commits": build_reference,
                    "deployment_units": deployment_unit,
                    "image_formats": image_format,
                    "code_tags": code_tag,
                    "registry_scopes": registry_scope,
                },
            },
        ]



from setuptools import setup, find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()

about = {}
with open("hamlet/__about__.py", "r") as fp:
    exec(fp.read(), about)

packages = find_packages(exclude=["tests.*", "tests"])

setup(
    name=about["__title__"],
    author=about["__author__"],
    url=about["__url__"],
    description=about["__summary__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="GPLv3",
    project_urls={"Repository": about["__repository_url__"]},
    packages=packages,
    setup_requires=["setuptools_scm>=7.1.0,<8.0.0"],
    use_scm_version={
        "root": "",
        "relative_to": __file__,
        "fallback_version": "_testing_",
        "write_to": "hamlet/__version__.py",
        "local_scheme": "no-local-version",
    },
    install_requires=[
        "click>=8.0.3,<9.0.0",
        "click-configfile>=0.2.3,<1.0.0",
        "cookiecutter>=2.1.1,<3.0.0",
        "tabulate>=0.8.0,<1.0.0",
        "Jinja2>=3.0.3,<4.0.0",
        "jmespath>=0.10.0,<1.0.0",
        "www-authenticate>=0.9.2,<1.0.0",
        "httpx>=0.21.2,<1.0.0",
        "marshmallow>=3.7.0,<4.0.0",
        "dulwich>=0.20.45,<1.0.0",
        "semver>=2.13.0,<3.0.0",
        "importlib_resources>=5.10.2,<6.0.0",
        # contract execution
        "boto3>=1.20.0,<2.0.0",
        "botocore>=1.20.0,<2.0.0",
        "fabric>=2.6.0,<3.0.0",
        "simple-term-menu>=1.4.1,<2.0.0",
        "docker>=5.0.3,<6.0.0",
        # Template testing
        "pytest>=6.0.0,<7.0.0",
    ],
    extras_require={
        "dev": [
            "coverage",
            "freezegun",
            "setuptools",
            # formatting
            "flake8",
            "black",
            # packaging
            "build",
            "wheel",
            "twine",
            # Docs
            "Sphinx",
            "sphinx-markdown-builder",
            "sphinx-click",
        ],
    },
    include_package_data=True,
    python_requires=">=3.7",
    entry_points={"console_scripts": ["hamlet=hamlet:command.root"]},
    classifiers=[
        "Natural Language :: English",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Java",
        "Programming Language :: Python",
    ],
)



from hamlet.backend.generate.cmdb import account, tenant, product
from .conftest import conf


def run(cmdb, clear_cmdb):
    clear_cmdb()
    with cmdb() as path:
        tenant.run(**conf["cmdb"]["tenant"], output_dir=path())
    with cmdb("accounts") as path:
        account.run(**conf["cmdb"]["account"], output_dir=path())
    with cmdb() as path:
        product.run(**conf["cmdb"]["product"], output_dir=path())



import os
import json
import shutil
import hashlib
import platform

from abc import ABC, abstractmethod

from hamlet.backend.common.exceptions import BackendException

from .engine_part import EnginePartInterface
from .engine_source import EngineSourceInterface
from .common import ENGINE_STATE_FILE_NAME, ENGINE_STATE_VERSION
from .exceptions import HamletEngineInvalidVersion, EngineStoreMissingEngineException


class EngineInterface(ABC):
    def __init__(self, name, description):
        self.name = name
        self.description = description
        self._engine_state_filename = ENGINE_STATE_FILE_NAME

        self._location = None
        self._digest = None

        self._engine_dir = None
        self._engine_state = {
            "name": self.name,
            "description": self.description,
        }

        self._parts = {}
        self._sources = {}

    @property
    def engine_dir(self):
        """
        The engine dir defines the base folder where engines are kept
        """
        return self._engine_dir

    @property
    def location(self):
        """
        The location of engine
        """
        return self._location

    @location.setter
    def location(self, location):
        """
        Set the location of this engine ( defined at load time )
        """
        self._location = location

    @engine_dir.setter
    def engine_dir(self, engine_dir):
        """
        The engine dir is set as part of the loader process
        """
        self._engine_dir = engine_dir
        self._load_engine_state()

    @property
    def install_path(self):
        """
        The install path is where the components of the engine will be installed
        """
        return os.path.join(self.engine_dir, self.name)

    @property
    def engine_state_file(self):
        """
        The install state file is a persistant file that holds the install_state
        """
        return os.path.join(self.install_path, self._engine_state_filename)

    @property
    def install_state(self):
        """
        The install state contains the details about the installation
        """
        return None

    def update_install_state(self, source_install_state=None):
        """
        Builds the internal state and saves it to the persistent store
        """
        source_digests = []
        source_install_details = {}
        if source_install_state is not None:
            source_digests = [s.digest for s in source_install_state]
            source_install_details = [vars(s) for s in source_install_state]

        install_state = {
            "part_paths": self._get_part_paths(),
            "digest": self._format_engine_digest(source_digests),
            "source_install_state": source_install_details,
        }

        self._engine_state["version"] = ENGINE_STATE_VERSION
        self._engine_state["install"] = install_state
        self._save_engine_state()

    @property
    def digest(self):
        """
        returns the digest of the engine
        """
        if self._digest is None:
            self._digest = self._format_engine_digest([s.digest for s in self.sources])

        return self._digest

    @property
    def short_digest(self):
        return self.digest[len("sha256:") : 15]

    @property
    def parts(self):
        """
        Parts link concrete hamlet components to sources
        """
        return self._parts.values()

    @parts.setter
    def parts(self, parts):
        for part in parts:
            if isinstance(part, EnginePartInterface):
                self._parts[part.type] = part
            else:
                raise ValueError("part is not an EnginePart")

    @parts.deleter
    def parts(self, type):
        try:
            del self._parts[type]
        except KeyError:
            raise BackendException(f"part {type} not found")

    @property
    def sources(self):
        """
        A source defines an external location which implements engine part(s)
        """
        return self._sources.values()

    @sources.setter
    def sources(self, sources):
        for source in sources:
            if isinstance(source, EngineSourceInterface):
                self._sources[source.name] = source
            else:
                raise ValueError("source is not an EngineSource")

    @sources.deleter
    def sources(self, source_name):
        try:
            del self._sources[source_name]
        except KeyError:
            raise BackendException(f"source {source_name} not found")

    @property
    def part_paths(self):
        """
        Part paths list the physical location of an installed part for a given engine
        """
        if self.install_state is not None:
            return self.install_state.get("part_paths", None)
        else:
            return {}

    @property
    def environment(self):
        """
        Using the part parts generate a dict of hamlet environment variables that are used
        to determine where different engine parts are by the executor
        """
        return {}

    def _get_source(self, source_name):
        """
        Get a particular source
        """
        source = None
        try:
            source = self._sources[source_name]
        except KeyError:
            raise BackendException(f"source {source_name} not found in engine")

        return source

    def _get_engine_source_dir(self, source_name):
        """
        Find the location of a source in an engines installation
        """
        source = self._get_source(source_name)
        return os.path.join(self.install_path, source.name)

    def _get_engine_env_dir(self, source_name):
        """
        Find the directory for the source that will be used in env var paths
        """
        source = self._get_source(source_name)
        if source.env_path is not None:
            return source.env_path
        else:
            return self._get_engine_source_dir(source_name)

    def _load_engine_state(self):
        """
        Load the state of an engine based on its install path
        """
        if os.path.isfile(self.engine_state_file):
            with open(self.engine_state_file, "r") as file:
                self._engine_state = json.load(file)
                engine_version = self._engine_state.get("version", "0.0.0")

                if engine_version != ENGINE_STATE_VERSION:
                    raise HamletEngineInvalidVersion(
                        engine_name=self.name,
                        version=engine_version,
                    )

    def _save_engine_state(self):
        """
        Writes the engine state to disk so that it can be accessed between cli runs
        """
        if not os.path.isdir(self.install_path):
            os.makedirs(self.install_path)

        with open(self.engine_state_file, "w") as file:
            json.dump(self._engine_state, file)

    def _get_part_paths(self):
        part_paths = {}
        for part in self.parts:
            part_paths[part.type] = os.path.join(
                self._get_engine_env_dir(part.source_name), part.source_path
            )
        return part_paths

    def _format_engine_digest(self, source_digests):
        return (
            "sha256:"
            + hashlib.sha256(":".join(source_digests).encode("utf-8")).hexdigest()
        )

    @abstractmethod
    def install(self):
        """
        The install method should loop through all sources and run their pull() method
        Once this has been completed the function should run self.update_install_state()
        This will collect the current state once the install has been completed
        """
        pass

    def set_default_engine(self, engine, engine_store):
        """
        A hook to run when an engine_store is set to a new default
        """
        pass


class InstalledEngine(EngineInterface):
    def __init__(self, name, description, digest, state_version):
        super().__init__(name, description)
        self._installed_digest = digest
        self._engine_state["version"] = state_version

    @property
    def environment(self):
        """
        Using the part parts generate a dict of hamlet environment variables that are used
        to determine where different engine parts are by the executor
        """

        _environment = {
            "GENERATION_ENGINE_DIR": [{"part_type": "core-engine", "env_path": ""}],
            "GENERATION_PLUGIN_DIRS": [
                {"part_type": "engine-plugin-aws", "env_path": ""},
                {"part_type": "engine-plugin-azure", "env_path": ""},
            ],
            "GENERATION_WRAPPER_LOCAL_JAVA": [
                {
                    "env_value": "false"
                    if "bundled-engine-wrapper" in self.part_paths
                    else "true"
                }
            ],
            "GENERATION_WRAPPER_SCRIPT_FILE": [
                {
                    "part_type": "bundled-engine-wrapper",
                    "env_path": f"freemarker-wrapper-{platform.system()}/bin/freemarker-wrapper"
                    if platform.system() != "Windows"
                    else f"freemarker-wrapper-{platform.system()}/bin/freemarker-wrapper.bat",
                }
            ],
            "GENERATION_WRAPPER_JAR_FILE": [
                {"part_type": "engine-wrapper", "env_path": "freemarker-wrapper.jar"}
            ],
            "GENERATION_BASE_DIR": [{"part_type": "executor-bash", "env_path": ""}],
            "GENERATION_DIR": [{"part_type": "executor-bash", "env_path": "cli"}],
            "AUTOMATION_DIR": [
                {"part_type": "executor-bash", "env_path": "automation/jenkins/aws"}
            ],
            "AUTOMATION_BASE_DIR": [
                {"part_type": "executor-bash", "env_path": "automation"}
            ],
        }

        env_result = {}
        self._load_engine_state()
        if self.part_paths is not None:
            for k, part_mappings in _environment.items():
                env_values = []
                for part_mapping in part_mappings:
                    if "env_value" in part_mapping:
                        env_values.append(part_mapping["env_value"])

                    if (
                        "part_type" in part_mapping
                        and self.part_paths.get(part_mapping["part_type"], None)
                        is not None
                    ):
                        env_values.append(
                            os.path.dirname(
                                os.path.join(
                                    self.part_paths[part_mapping["part_type"]],
                                    part_mapping["env_path"],
                                )
                                + "/"
                            )
                        )
                env_result[k] = ";".join(env_values)
        return env_result

    @property
    def digest(self):
        return self._installed_digest

    def install(self):
        self.update_install_state()

    @property
    def install_state(self):
        """
        The install state contains the details about the installation
        """
        self._load_engine_state()
        return self._engine_state.get("install", None)


class ShimEngine(EngineInterface):
    def set_default_engine(self, engine, engine_store):

        try:
            installed_shim = engine_store.get_engine(self.name, locations=["installed"])
        except EngineStoreMissingEngineException:
            return

        for type, path in installed_shim.part_paths.items():
            if os.path.islink(path):
                os.unlink(path)

            if os.path.isdir(path):
                os.rmdir(path)

            if engine.part_paths.get(type, None):
                os.symlink(engine.part_paths[type], path, target_is_directory=True)
            else:
                os.makedirs(path)

    def install(self):

        source_details = []
        for source in self.sources:
            source_dir = self._get_engine_source_dir(source.name)
            source_details.append(source.pull(source_dir))
        self.update_install_state(source_details)


class Engine(EngineInterface):
    def install(self):

        if os.path.isdir(self.install_path):
            shutil.rmtree(self.install_path)

        source_details = []
        for source in self.sources:
            source_dir = self._get_engine_source_dir(source.name)
            source_details.append(source.pull(source_dir))

        self.update_install_state(source_details)



import tempfile
from configparser import ConfigParser
from hamlet.backend.common import aws_credentials_session
from hamlet.backend.common import runner


def run(AccountId, ProviderId, Provider, env={}):
    """
    Uses the standard hamlet credentials process to determine the
    authentication for this call

    For AWS collect the temporary credentials from an assume role and return
    as a credential set
    """

    cred_result = {}

    with tempfile.NamedTemporaryFile() as output_file:
        env = {
            **env,
            "CRED_ACCOUNT": AccountId,
            "PROVIDERID": ProviderId,
            "ACCOUNT_PROVIDER": Provider,
        }
        runner.run(
            "setCredentials.sh",
            args=[],
            options={},
            env=env,
            engine=None,
            _is_cli=True,
            script_base_path_env="AUTOMATION_DIR",
            extra_script=f"&&  env | grep 'AWS' > {output_file.name}",
        )

        config = ConfigParser(strict=False)

        s_config = open(output_file.name, "r").read()
        s_config = f"[ini]\n{s_config}"

        config.read_string(s_config)
        items = config.items("ini")
        for key, value in items:
            cred_result[key] = value

        existing_creds = None

        if cred_result.get("aws_access_key_id", None) is not None:
            existing_creds = {
                "aws_access_key_id": cred_result["aws_access_key_id"],
                "aws_secret_access_key": cred_result["aws_secret_access_key"],
                "aws_session_token": cred_result.get("aws_Session_token", None),
            }

        credentials = aws_credentials_session.boto3_session(
            profile_name=cred_result.get("aws_profile", None),
            config_file=cred_result.get("aws_config_file", None),
            credentials_file=cred_result.get("aws_shared_credentials_file", None),
            existing_creds=existing_creds,
        ).get_credentials()

        try:
            credentials = credentials.get_frozen_credentials()

        except AttributeError:
            pass

    return {
        "Properties": {
            "aws_access_key_id": credentials.access_key,
            "aws_secret_access_key": credentials.secret_key,
            "aws_session_token": credentials.token,
        }
    }



import os
import click
import re
import tempfile

from tabulate import tabulate

from hamlet.command import root as cli
from hamlet.command.common.display import json_or_table_option, wrap_text
from hamlet.command.common import exceptions
from hamlet.command.common.config import pass_options
from hamlet.backend.create import template as create_template_backend
from hamlet.backend.draw import diagram as create_diagram_backend
from hamlet.backend import query as query_backend


LIST_DIAGRAMS_QUERY = (
    "Diagrams[]" ".{" "Id:Id," "Type:Type," "Description:Description" "}"
)


def find_diagrams_from_options(options, ids):
    query_args = {
        **options.opts,
        "generation_entrance": "diagraminfo",
        "output_filename": "diagraminfo.json",
    }
    available_diagrams = query_backend.run(
        **query_args,
        cwd=os.getcwd(),
        engine=options.engine,
        query_text=LIST_DIAGRAMS_QUERY,
    )

    diagrams = []

    for diagram in available_diagrams:
        for id in ids:
            if re.fullmatch(id, diagram["Id"]):
                diagrams.append(diagram)

    return diagrams


@cli.group("visual", context_settings=dict(max_content_width=240))
def group():
    """
    Generates visual representations of your hamlet
    """
    pass


def diagrams_table(data):
    tablerows = []
    for row in data:
        tablerows.append(
            [
                wrap_text(row["Id"]),
                wrap_text(row["Type"]),
                wrap_text(row["Description"]),
            ]
        )
    return tabulate(
        tablerows,
        headers=["Id", "Type", "Description"],
        showindex=True,
        tablefmt="fancy_grid",
    )


@group.command(
    "list-diagrams", short_help="", context_settings=dict(max_content_width=240)
)
@click.option(
    "-i",
    "--diagram-id",
    default=[".*"],
    show_default=True,
    multiple=True,
    help="The deployment id pattern to match",
)
@json_or_table_option(diagrams_table)
@exceptions.backend_handler()
@pass_options
def list_diagrams(options, diagram_id):
    """
    Lists the diagrams available to create
    """
    return find_diagrams_from_options(options, diagram_id)


@group.command(
    "draw-diagrams", short_help="", context_settings=dict(max_content_width=240)
)
@click.option(
    "-s",
    "--src-dir",
    required=False,
    type=click.Path(
        file_okay=False,
        dir_okay=True,
        writable=True,
        readable=True,
    ),
    default=None,
    help="the directory to save the generation scripts to - temp dir by default",
)
@click.option(
    "-d",
    "--asset-dir",
    required=True,
    type=click.Path(
        file_okay=False,
        dir_okay=True,
        writable=True,
        readable=True,
    ),
    help="the directory to save the generated diagrams to",
)
@click.option(
    "-i",
    "--diagram-id",
    default=[".*"],
    show_default=True,
    multiple=True,
    help="The diagram id pattern to match",
)
@exceptions.backend_handler()
@pass_options
def draw_diagrams(options, diagram_id, src_dir, asset_dir):
    """
    Draw a collection of digrams based on your solution
    """
    temp_dir = None

    if src_dir is None:
        temp_dir = tempfile.TemporaryDirectory()
        src_dir = temp_dir.name

    diagrams = find_diagrams_from_options(options, diagram_id)

    if len(diagrams) == 0:
        raise exceptions.CommandError("No diagrams found that match pattern")

    for diagram in diagrams:

        diagram_id = diagram["Id"]

        click.echo((click.style(f"[*] {diagram_id}", bold=True, fg="green")))
        args = {
            **options.opts,
            "entrance": "diagram",
            "deployment_unit": diagram_id,
            "output_dir": src_dir,
        }
        create_template_backend.run(**args, engine=options.engine, _is_cli=False)
        create_diagram_backend.run(
            diagram_id=diagram_id,
            src_dir=src_dir,
            output_dir=asset_dir,
            engine=options.engine,
        )

    if temp_dir is not None:
        temp_dir.cleanup()


LIST_DIAGRAM_TYPES_QUERY = (
    "DiagramTypes[]" ".{" "Type:Type," "Description:Description" "}"
)


def diagram_types_table(data):
    tablerows = []
    for row in data:
        tablerows.append(
            [
                wrap_text(row["Type"]),
                wrap_text(row["Description"]),
            ]
        )
    return tabulate(
        tablerows,
        headers=["Type", "Description"],
        showindex=True,
        tablefmt="fancy_grid",
    )


@group.command(
    "list-diagram-types", short_help="", context_settings=dict(max_content_width=240)
)
@json_or_table_option(diagram_types_table)
@exceptions.backend_handler()
@pass_options
def list_diagram_types(options):
    """
    Lists the types of diagrams available
    """
    args = {
        **options.opts,
        "generation_input_source": "mock",
        "generation_entrance": "diagraminfo",
        "output_filename": "diagraminfo.json",
    }

    return query_backend.run(
        **args,
        cwd=os.getcwd(),
        engine=options.engine,
        query_text=LIST_DIAGRAM_TYPES_QUERY,
    )



import logging
from logging.config import dictConfig


LOGGING = {
    "version": 1,
    "disableExistingLoggers": False,
    "formatters": {"default": {"format": "[%(levelname)s] %(name)s: %(message)s"}},
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "default",
        },
    },
    "loggers": {
        "": {
            "level": "INFO",
            "handlers": ["console"],
        }
    },
}


dictConfig(LOGGING)

root = logging.getLogger()



import os
import re

from hamlet.backend import query as query_backend
from hamlet.backend.manage import stack as manage_stack_backend
from hamlet.backend.manage import deployment as manage_deployment_backend
from hamlet.backend.create import template as create_template_backend
from hamlet.backend.common.exceptions import BackendException


class UnsupportedDeploymentProviderException(BackendException):
    def __init__(self, deployment_provider, message):
        self.deployment_provider = deployment_provider
        super.__init__(message=message)


LIST_DEPLOYMENTS_QUERY = (
    "Stages[].Steps[]"
    ".{"
    "DeploymentGroup:Parameters.DeploymentGroup,"
    "DeploymentUnit:Parameters.DeploymentUnit,"
    "DeploymentProvider:Parameters.DeploymentProvider,"
    "Operations:Parameters.Operations,"
    "CurrentState:Parameters.CurrentState"
    "}"
)


def find_deployments(
    deployment_mode,
    deployment_group,
    deployment_units,
    engine,
    deployment_states=["deployed", "notdeployed"],
    **kwargs,
):

    query_args = {
        **kwargs,
        "deployment_mode": deployment_mode,
        "generation_entrance": "unitlist",
        "output_filename": "unitlist-managementcontract.json",
    }
    available_deployments = query_backend.run(
        **query_args,
        cwd=os.getcwd(),
        query_text=LIST_DEPLOYMENTS_QUERY,
        engine=engine,
    )

    deployments = []

    for deployment in available_deployments:
        if re.fullmatch(deployment_group, deployment["DeploymentGroup"]):
            for deployment_unit in deployment_units:
                if re.fullmatch(deployment_unit, deployment["DeploymentUnit"]):
                    if deployment["CurrentState"] in deployment_states:
                        deployments.append(deployment)

    return deployments


def create_deployment(
    deployment_group,
    deployment_unit,
    deployment_mode,
    output_dir,
    engine,
    _is_cli=True,
    **kwargs,
):

    generate_args = {
        **kwargs,
        "entrance": "deployment",
        "deployment_group": deployment_group,
        "deployment_unit": deployment_unit,
        "deployment_mode": deployment_mode,
        "output_dir": output_dir,
    }
    create_template_backend.run(**generate_args, engine=engine, _is_cli=_is_cli)


def run_deployment(
    deployment_provider,
    deployment_group,
    deployment_unit,
    operation,
    output_dir,
    engine,
    dryrun=False,
    **kwargs,
):

    manage_args = {
        "deployment_group": deployment_group,
        "deployment_unit": deployment_unit,
        "output_dir": output_dir,
        "dryrun": dryrun,
        **kwargs,
    }

    if operation == "delete":
        manage_args["delete"] = True

    if deployment_provider == "aws":
        manage_stack_backend.run(**manage_args, engine=engine, _is_cli=True)

    elif deployment_provider == "azure":
        manage_deployment_backend.run(**manage_args, engine=engine, _is_cli=True)

    else:
        raise UnsupportedDeploymentProviderException(
            f"Deployment provider {deployment_provider} is not supported",
            deployment_provider=deployment_provider,
        )



import collections
from unittest import mock
from click.testing import CliRunner
from hamlet.command.generate.cmdb import generate_account
from tests.unit.command.generate.test_generate_command import run_generate_command_test

OPTIONS = collections.OrderedDict()
OPTIONS["!--account-id,account_id"] = "test-account-id"
OPTIONS["!--provider-id,provider_id"] = "test-provider-id"
OPTIONS["--account-name,account_name"] = ("test-account-name", "test-account-id")
OPTIONS["--account-seed,account_seed"] = ("not-random-seed", "random-seed")
OPTIONS["--provider-type,provider_type"] = ("aws", "azure")


@mock.patch("hamlet.command.generate.cmdb.generate_account_backend")
def test(generate_account_backend):
    generate_account_backend.generate_account_seed.return_value = "random-seed"
    run_generate_command_test(
        CliRunner(), generate_account, generate_account_backend.run, OPTIONS
    )



import os
import json
from unittest import mock
from hamlet.backend.deploy import create_deployment as create_deployment_backend
from .conftest import conf


ACCOUNT = conf["cmdb"]["account"]["account_name"]
LEVEL = "account"


#  test is made accordingly to quickstart guide
@mock.patch.dict(os.environ, {"ACCOUNT": ACCOUNT, "DISTRICT_TYPE": "account"})
def run(cmdb, engine):

    with mock.patch.dict(os.environ, {"ROOT_DIR": cmdb.ROOT_DIR}):

        create_deployment_backend(
            deployment_group=LEVEL,
            deployment_unit="audit",
            deployment_mode="update",
            output_dir=None,
            engine=engine.engine,
        )
        create_deployment_backend(
            deployment_group=LEVEL,
            deployment_unit="s3",
            deployment_mode="update",
            output_dir=None,
            engine=engine.engine,
        )

        #  check that files were created
        with cmdb(
            "accounts", ACCOUNT, "infrastructure", "cf", "shared", "s3", "default"
        ) as filename:

            def prefix(level):
                return [
                    LEVEL,
                    level,
                    ACCOUNT,
                    conf["cmdb"]["tenant"]["default_region"],
                ]

            assert os.path.exists(
                filename(*prefix("s3"), "epilogue", sep="-", ext="sh")
            )

            assert os.path.exists(
                filename(*prefix("s3"), "generation-contract", sep="-", ext="json")
            )

            cf_s3_template_filename = filename(
                *prefix("s3"), "template", sep="-", ext="json"
            )

            assert os.path.exists(cf_s3_template_filename)
            with open(cf_s3_template_filename) as f:
                json.load(f)

        with cmdb(
            "accounts", ACCOUNT, "infrastructure", "cf", "shared", "audit", "default"
        ) as filename:

            def prefix(level):
                return [
                    LEVEL,
                    level,
                    ACCOUNT,
                    conf["cmdb"]["tenant"]["default_region"],
                ]

            assert os.path.exists(
                filename(*prefix("audit"), "generation-contract", sep="-", ext="json")
            )

            cf_audit_template_filename = filename(
                *prefix("audit"), "template", sep="-", ext="json"
            )

            assert os.path.exists(cf_audit_template_filename)
            with open(cf_audit_template_filename) as f:
                json.load(f)



import os
import json

from unittest import mock
from click.testing import CliRunner
from hamlet.command.schema import list_schemas
from hamlet.command.common.config import Options


def template_backend_run_mock(data):
    def run(
        output_filename="schemalist-schemacontract.json",
        output_dir=None,
        *args,
        **kwargs
    ):
        os.makedirs(output_dir, exist_ok=True)
        unitlist_filename = os.path.join(output_dir, output_filename)
        with open(unitlist_filename, "wt+") as f:
            json.dump(data, f)

    return run


def mock_backend(schemalist=None):
    def decorator(func):
        @mock.patch("hamlet.backend.query.template")
        def wrapper(blueprint_mock, *args, **kwargs):
            blueprint_mock.run.side_effect = template_backend_run_mock(schemalist)
            return func(blueprint_mock, *args, **kwargs)

        return wrapper

    return decorator


@mock_backend(
    {
        "Stages": [
            {
                "Id": "StageId1",
                "Steps": [
                    {
                        "Id": "StepId1",
                        "Parameters": {"Schema": "Schema1"},
                    },
                    {
                        "Id": "StepId2",
                        "Parameters": {"Schema": "Schema2"},
                    },
                ],
            },
            {
                "Id": "StageId2",
                "Steps": [
                    {
                        "Id": "StepId3",
                        "Parameters": {"Schema": "Schema3"},
                    },
                    {
                        "Id": "StepId4",
                        "Parameters": {"Schema": "Schema4"},
                    },
                ],
            },
        ]
    }
)
def test_query_list_schemas(blueprint_mock):
    obj = Options()

    cli = CliRunner()
    result = cli.invoke(list_schemas, ["--output-format", "json"], obj=obj)
    print(result.output)
    assert result.exit_code == 0
    result = json.loads(result.output)
    assert len(result) == 4
    assert {
        "Schema": "Schema1",
    } in result
    assert {
        "Schema": "Schema2",
    } in result
    assert {
        "Schema": "Schema3",
    } in result
    assert {
        "Schema": "Schema4",
    } in result



import click
import functools
import os

from tabulate import tabulate

from hamlet.command import root as cli
from hamlet.command.common import exceptions, config
from hamlet.command.common.display import json_or_table_option, wrap_text

from hamlet.backend.automation_tasks import upload_image as upload_image_backend
from hamlet.backend.automation_tasks import transfer_image as transfer_image_backend
from hamlet.backend.automation_tasks import update_build as update_build_backend
from hamlet.backend import query as query_backend


def query_imagedetails_output(options, query, query_params=None, sub_query_text=None):
    query_args = {
        **options.opts,
        "generation_entrance": "imagedetails",
        "output_filename": "imagedetails-config.json",
    }
    query_result = query_backend.run(
        **query_args,
        cwd=os.getcwd(),
        query_text=query,
        query_params=query_params,
        sub_query_text=sub_query_text,
        engine=options.engine
    )

    return query_result


def image_options(func):
    """Standard image details"""

    @click.option(
        "-u",
        "--deployment-unit",
        required=True,
        help="The deployment unit the image belongs to",
    )
    @click.option(
        "-r",
        "--build-reference",
        required=True,
        help="The unique reference for the build of this image - usually git commit",
    )
    @click.option(
        "--code-tag",
        help="A tag applied to your code repo which will be mapped to a build reference",
    )
    @click.option(
        "-f",
        "--image-format",
        help="The format of the code image",
        type=click.Choice(
            [
                "dataset",
                "rdssnapshot",
                "docker",
                "lambda",
                "lambda_jar",
                "pipeline",
                "scripts",
                "openapi",
                "swagger",
                "spa",
                "contentnode",
            ]
        ),
    )
    @click.option(
        "-s",
        "--registry-scope",
        help="The scope of the registry to update the image to",
    )
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        """
        Image Detail handling
        """
        return func(*args, **kwargs)

    return wrapper


@cli.group("release", context_settings=dict(max_content_width=240))
def group():
    """
    Manage the lifecycle of images for occurrences
    """


@group.command(
    "upload-image", short_help="", context_settings=dict(max_content_width=240)
)
@image_options
@click.option(
    "--image-path",
    help="The path to the image can be zip file or directory",
    type=click.Path(file_okay=True, dir_okay=True, readable=True, resolve_path=True),
)
@click.option(
    "--dockerfile",
    help="The path to a dockerfile to create the image with",
    type=click.Path(file_okay=True, dir_okay=False, readable=True, resolve_path=True),
)
@click.option(
    "--docker-context",
    help="The docker context directory used with the dockerfile",
    type=click.Path(file_okay=False, dir_okay=True, readable=True, resolve_path=True),
)
@click.option("--docker-image", help="The tag of an existing docker image")
@exceptions.backend_handler()
@config.pass_options
def upload_image(options, **kwargs):
    """
    Upload a code image to a registry
    """
    task = upload_image_backend.UploadImageAutomationRunner(
        **options.opts, **kwargs, engine=options.engine
    )
    task.run()


@group.command(
    "transfer-image", short_help="", context_settings=dict(max_content_width=240)
)
@image_options
@click.option(
    "--source-account",
    required=True,
    help="The name of the account to get the image from",
)
@click.option(
    "--source-environment",
    required=True,
    help="The name of the environment to get the image from",
)
@exceptions.backend_handler()
@config.pass_options
def transfer_image(options, **kwargs):
    """
    Transfer an image between registries
    """
    task = transfer_image_backend.TransferImageAutomationRunner(
        **options.opts, **kwargs, engine=options.engine
    )
    task.run()


@group.command(
    "update-image-reference",
    short_help="",
    context_settings=dict(max_content_width=240),
)
@image_options
@exceptions.backend_handler()
@config.pass_options
def update_image_reference(options, **kwargs):
    """
    Update the image reference for a component
    """
    task = update_build_backend.UpdateBuildAutomationRunner(
        **options.opts, **kwargs, engine=options.engine
    )
    task.run()


def image_references_table(data):
    tablerows = []
    for row in data:
        tablerows.append(
            [
                wrap_text(row["Occurrence"]),
                wrap_text(row["BUILD_UNIT"]),
                wrap_text(row["BUILD_REFERENCE"]),
                wrap_text(row["APP_REFERENCE"]),
                wrap_text(row["BUILD_FORMATS"]),
                wrap_text(row["BUILD_SOURCE"]),
            ]
        )
    return tabulate(
        tablerows,
        headers=[
            "Occurrence",
            "Deployment Unit",
            "Build Reference",
            "Code Tag",
            "Image Format",
            "Image Source",
        ],
        tablefmt="github",
    )


@group.command(
    "list-image-references",
    short_help="",
    context_settings=dict(max_content_width=240),
)
@click.option("-q", "--query", help="A query to filter out the results")
@json_or_table_option(image_references_table)
@exceptions.backend_handler()
@config.pass_options
def list_image_references(options, query):
    """
    List the references for all components
    """

    LIST_IMAGE_REFERENCES_QUERY = (
        "Images[]"
        ".{"
        "Occurrence:Occurrence,"
        "BUILD_FORMATS:BUILD_FORMATS,"
        "BUILD_UNIT:BUILD_UNIT,"
        "BUILD_REFERENCE:BUILD_REFERENCE,"
        "APP_REFERENCE:APP_REFERENCE,"
        "BUILD_SOURCE:BUILD_SOURCE"
        "}"
    )

    return query_imagedetails_output(options, LIST_IMAGE_REFERENCES_QUERY, None, query)



import collections
import json
import os

from unittest import mock
from click.testing import CliRunner
from hamlet.command.task import run_runbook
from tests.unit.command.test_option_generation import (
    run_options_test,
    run_validatable_option_test,
)

ALL_VALID_OPTIONS = collections.OrderedDict()
ALL_VALID_OPTIONS["!-n,--name"] = "RunBook1"
ALL_VALID_OPTIONS["--confirm"] = False
ALL_VALID_OPTIONS["--silent"] = False
ALL_VALID_OPTIONS["--"] = "test=true"


def template_backend_run_mock(data):
    def run(output_filename=None, output_dir=None, *args, **kwargs):
        for set in data:
            os.makedirs(output_dir, exist_ok=True)
            filename = os.path.join(output_dir, set[2])
            with open(filename, "wt+") as f:
                json.dump(set[0], f)

    return run


def mock_backend(runbookinfo=None, contract=None):
    def decorator(func):
        @mock.patch("hamlet.command.task.contract_backend")
        @mock.patch("hamlet.backend.query.template")
        def wrapper(blueprint_mock, contract_backend, *args, **kwargs):
            blueprint_mock.run.side_effect = template_backend_run_mock(
                [
                    (runbookinfo, "runbookinfo", "runbookinfo-config.json"),
                    (contract, "runbook", "runbook-contract.json"),
                ]
            )

            return func(blueprint_mock, contract_backend, *args, **kwargs)

        return wrapper

    return decorator


runbookinfo_mock_output = {
    "RunBooks": [
        {
            "Name": "RunBook1",
            "Description": "RunBookDescription1",
            "Engine": "RunBookEngine1",
        },
        {
            "Name": "RunBook2",
            "Description": "RunBookDescription2",
            "Engine": "RunBookEngine2",
        },
    ],
}

runbook_mock_output = {"Stages": [{"Id": "Stage1", "Steps": [{"Id": "Step1"}]}]}


@mock_backend(runbookinfo_mock_output, runbook_mock_output)
def test_input_valid(blueprint_mock, contract_backend):
    run_options_test(CliRunner(), run_runbook, ALL_VALID_OPTIONS, contract_backend.run)


@mock_backend(runbookinfo_mock_output, runbook_mock_output)
def test_input_validation(blueprint_mock, contract_backend):
    runner = CliRunner()
    run_validatable_option_test(
        runner,
        run_runbook,
        contract_backend.run,
        {
            "-n": "RunBook1",
        },
        [],
    )



import os
import click
from click.types import StringParamType
import functools

from hamlet.command.common.config import Options


def get_home_dir_default(subdir=""):
    return os.path.join(
        click.get_app_dir(app_name="hamlet", force_posix=True, roaming=False), subdir
    )


class CommaSplitParamType(StringParamType):
    envvar_list_splitter = ","

    def __repr__(self):
        return "STRING"


def common_cli_config_options(func):
    """Add common CLI config options to commands"""

    @click.option(
        "-p",
        "--profile",
        default=None,
        envvar="HAMLET_PROFILE",
        help="The name of the profile to use for configuration",
        show_envvar=True,
    )
    @click.option(
        "--hamlet-home-dir",
        type=click.Path(file_okay=False, dir_okay=True, readable=True, writable=True),
        envvar="HAMLET_HOME_DIR",
        default=get_home_dir_default(),
        help="The home directory used by hamlet",
        show_default=True,
        show_envvar=True,
    )
    @click.option(
        "--cli-config-dir",
        type=click.Path(file_okay=False, dir_okay=True, readable=True),
        envvar="HAMLET_CLI_CONFIG_DIR",
        default=get_home_dir_default("config"),
        help="The directory where profile configuration is stored",
        show_default=True,
        show_envvar=True,
    )
    @click.pass_context
    @functools.wraps(func)
    def wrapper(ctx, *args, **kwargs):
        """
        Config file handling
        """
        opts = ctx.ensure_object(Options)
        opts.hamlet_home_dir = kwargs.pop("hamlet_home_dir")
        opts.load_config_file(
            profile=kwargs.pop("profile"), searchpath=kwargs.pop("cli_config_dir")
        )

        kwargs["opts"] = opts
        return ctx.invoke(func, *args, **kwargs)

    return wrapper


def common_logging_options(func):
    """Add common options for logging"""

    @click.option(
        "--log-level",
        envvar="GENERATION_LOG_LEVEL",
        type=click.Choice(
            ["fatal", "error", "warn", "info", "debug", "trace"], case_sensitive=False
        ),
        default="info",
        help="The minimum log event level",
        show_default=True,
        show_envvar=True,
    )
    @click.option(
        "--log-format",
        envvar="GENERATION_LOG_FORMAT",
        type=click.Choice(["compact", "full"], case_sensitive=False),
        default="compact",
        help="The format used for engine log messages",
        show_default=True,
        show_envvar=True,
    )
    @click.pass_context
    @functools.wraps(func)
    def wrapper(ctx, *args, **kwargs):
        """
        Logging Options for the command line
        """
        opts = ctx.ensure_object(Options)
        opts.log_level = kwargs.pop("log_level")
        opts.log_format = kwargs.pop("log_format")

        kwargs["opts"] = opts
        return ctx.invoke(func, *args, **kwargs)

    return wrapper


def common_engine_options(func):
    """Add common options for the engine"""

    @click.option(
        "--engine",
        envvar="HAMLET_ENGINE",
        help="The name of the engine to use",
        show_envvar=True,
    )
    @click.option(
        "--engine-dir",
        type=click.Path(
            dir_okay=True,
            file_okay=False,
            writable=True,
            readable=True,
        ),
        default=get_home_dir_default("engine"),
        envvar="HAMLET_ENGINE_DIR",
        help="The location of the hamlet engine store",
        show_default=True,
        show_envvar=True,
    )
    @click.option(
        "--engine-config-dir",
        type=click.Path(
            dir_okay=True,
            file_okay=False,
            writable=True,
            readable=True,
        ),
        default=get_home_dir_default("config"),
        envvar="HAMLET_ENGINE_CONFIG",
        help="The location of the hamlet engine config file for local engines",
        show_default=True,
        show_envvar=True,
    )
    @click.option(
        "--engine-search-locations",
        multiple=True,
        default=["installed", "local", "remote"],
        type=click.Choice(["installed", "local", "remote", "hidden"]),
        show_default=True,
    )
    @click.pass_context
    @functools.wraps(func)
    def wrapper(ctx, *args, **kwargs):
        """
        Engine configuration options
        """
        opts = ctx.ensure_object(Options)
        opts.set_engine_store(
            kwargs.pop("engine_dir"), [kwargs.pop("engine_config_dir")]
        )
        opts.set_engine(kwargs.pop("engine"), kwargs.pop("engine_search_locations"))
        return ctx.invoke(func, *args, **kwargs)

    return wrapper


def common_generation_options(func):
    """Add common options for generation"""

    @click.option(
        "-p",
        "--generation-provider",
        envvar="GENERATION_PROVIDERS",
        help="plugins to load for output generation",
        default=["aws"],
        type=CommaSplitParamType(),
        multiple=True,
        show_default=True,
        show_envvar=True,
    )
    @click.option(
        "-f",
        "--generation-framework",
        help="output framework to use for output generation",
        default="cf",
        show_default=True,
    )
    @click.option(
        "-i",
        "--generation-input-source",
        help="source of input data to use when generating the output",
        default="composite",
        show_default=True,
    )
    @click.pass_context
    @functools.wraps(func)
    def wrapper(ctx, *args, **kwargs):
        """
        Logging Options for the command line
        """
        opts = ctx.ensure_object(Options)
        opts.generation_provider = kwargs.pop("generation_provider")
        opts.generation_framework = kwargs.pop("generation_framework")
        opts.generation_input_source = kwargs.pop("generation_input_source")

        kwargs["opts"] = opts
        return ctx.invoke(func, *args, **kwargs)

    return wrapper


def common_district_options(func):
    """Add Common options for district config"""

    @click.option(
        "--root-dir",
        envvar="ROOT_DIR",
        help="The root CMDB directory",
        show_envvar=True,
        type=click.Path(
            file_okay=False,
            dir_okay=True,
            writable=True,
            resolve_path=True,
            allow_dash=False,
        ),
    )
    @click.option(
        "--district-type",
        envvar="DISTRICT_TYPE",
        help="The type of district to target",
        default="segment",
        show_envvar=True,
    )
    @click.option(
        "--tenant",
        envvar="TENANT",
        help="The tenant name to use",
        show_envvar=True,
    )
    @click.option(
        "--account",
        envvar="ACCOUNT",
        help="The account name to use",
        show_envvar=True,
    )
    @click.option(
        "--product",
        envvar="PRODUCT",
        help="The product name to use",
        show_envvar=True,
    )
    @click.option(
        "--environment",
        envvar="ENVIRONMENT",
        help="The environment name to use",
        show_envvar=True,
    )
    @click.option(
        "--segment",
        envvar="SEGMENT",
        help="The segment name to use",
        show_envvar=True,
    )
    @click.pass_context
    @functools.wraps(func)
    def wrapper(ctx, *args, **kwargs):
        """
        District options from cmd line or file
        """
        opts = ctx.ensure_object(Options)
        opts.root_dir = kwargs.pop("root_dir")
        opts.district_type = kwargs.pop("district_type")
        opts.tenant = kwargs.pop("tenant")
        opts.account = kwargs.pop("account")
        opts.product = kwargs.pop("product")
        opts.environment = kwargs.pop("environment")
        opts.segment = kwargs.pop("segment")

        kwargs["opts"] = opts
        return ctx.invoke(func, *args, **kwargs)

    return wrapper



import boto3


def run(
    BucketName,
    Prefix,
    Region=None,
    AWSAccessKeyId=None,
    AWSSecretAccessKey=None,
    AWSSessionToken=None,
    env={},
):
    """
    Remove all objects from an S3 bucket prefix
    """

    session = boto3.Session(
        aws_access_key_id=AWSAccessKeyId,
        aws_secret_access_key=AWSSecretAccessKey,
        aws_session_token=AWSSessionToken,
        region_name=Region,
    )

    s3 = session.resource("s3")
    bucket = s3.Bucket(BucketName)
    object_count = 0

    if bucket.Versioning().status in ["Enabled", "Suspended"]:
        for object in bucket.object_versions.filter(Prefix=Prefix):
            object_count += 1
            object.delete()

    else:
        for object in bucket.objects.filter(Prefix=Prefix):
            object_count += 1
            object.delete()

    return {
        "Properties": {
            "object_count": str(object_count),
            "s3_path": f"s3://{BucketName}/" + Prefix,
        }
    }



import click

from cookiecutter.exceptions import OutputDirExistsException

from hamlet.backend.generate.cmdb import account as generate_account_backend
from hamlet.backend.generate.cmdb import tenant as generate_tenant_backend
from hamlet.backend.generate.cmdb import product as generate_product_backend
from hamlet.utils import dynamic_option, DynamicCommand
from hamlet.command.generate import decorators
from hamlet.command.common.exceptions import CommandError


@click.command(
    "tenant-cmdb", cls=DynamicCommand, context_settings=dict(max_content_width=240)
)
@dynamic_option(
    "--tenant-id",
    help="The unique Id for the tenant",
    prompt=True,
    prompt_required=True,
    required=True,
)
@dynamic_option(
    "--tenant-name",
    help="A more descriptive name for the tenant",
    default=lambda p: p.tenant_id,
)
@dynamic_option(
    "--default-region",
    help="The id of the default region to use for deployments in this tenant",
    default="ap-southeast-2",
)
@dynamic_option(
    "--audit-log-expiry-days",
    help="How long to keep account audit logs for",
    type=click.INT,
    default=2555,
)
@dynamic_option(
    "--audit-log-offline-days",
    help="How log until audit logs are sent to archive storage",
    type=click.INT,
    default=90,
)
@decorators.common_generate_options
def generate_tenant(**kwargs):
    """
    Creates a tenant cmdb.
    This should be run from the root of an accounts repository.
    """
    try:
        generate_tenant_backend.run(**kwargs)

    except OutputDirExistsException as e:
        raise CommandError(e)


@click.command(
    "account-cmdb", cls=DynamicCommand, context_settings=dict(max_content_width=240)
)
@dynamic_option(
    "--account-id",
    help="The unique id for the account",
    prompt=True,
    prompt_required=False,
    required=True,
)
@dynamic_option(
    "--account-name",
    help="A more descriptive name for the account",
    default=lambda p: p.account_id,
)
@dynamic_option(
    "--account-seed",
    help="A random seed to ensure unique deployments",
    default=lambda p: generate_account_backend.generate_account_seed(),
)
@dynamic_option(
    "--provider-type",
    help="The cloud provider the account represents",
    type=click.Choice(
        [
            "aws",
            "azure",
        ]
    ),
    default="aws",
)
@dynamic_option(
    "--provider-id",
    help="The cloud provider Id for for the account (AWS Account Id or Azure Subscription)",
    required=True,
    prompt=True,
    prompt_required=False,
)
@decorators.common_generate_options
def generate_account(**kwargs):
    """
    Creates an account cmdb within a tenant cmdb.

    The account template can be reused to create multiple accounts within a Tenant.
    This template should be run from a tenant directory
    """
    try:
        generate_account_backend.run(**kwargs)

    except OutputDirExistsException as e:
        raise CommandError(e)


@click.command(
    "product-cmdb", cls=DynamicCommand, context_settings=dict(max_content_width=240)
)
@dynamic_option(
    "--product-id",
    help="The Id of your product",
    prompt=True,
    prompt_required=False,
    required=True,
)
@dynamic_option(
    "--dns-zone",
    help="A DNS zone name which will act as a base domain for public component hostnames",
    default="",
)
@dynamic_option(
    "--product-name",
    help="A more descriptive name of your product",
    default=lambda p: p.product_id,
)
@dynamic_option(
    "--environment-id",
    help="The id of your first deployed environment",
    default="int",
)
@dynamic_option(
    "--environment-name",
    help="A more descriptive name of your environment",
    default="integration",
)
@dynamic_option(
    "--segment-id",
    help="The id of the first segment in your environments",
    default="default",
)
@dynamic_option(
    "--segment-name",
    help="A more descriptive name of your segment",
    default=lambda p: p.segment_id,
)
@decorators.common_generate_options
def generate_product(**kwargs):
    """
    Creates a product cmdb.
    This template should be run from the root of an empty product directory.
    """
    try:
        generate_product_backend.run(**kwargs)
    except OutputDirExistsException as e:
        raise CommandError(e)



from hamlet.backend.common.exceptions import BackendException


def run(FilePath, env={}):
    """
    read the content of a given filepath and return it as the result
    """
    content = ""
    try:
        with open(FilePath, "r") as file:
            content = file.read()

    except FileNotFoundError as e:
        raise BackendException(str(e))

    return {"Properties": {"result": content}}



import click
import os

from click_configfile import Param


class DynamicCommand(click.Command):
    def invoke(self, ctx):
        if self.callback is not None:
            for param in self.params:
                try:
                    ctx.params[param.name]
                except KeyError:
                    if param.default is not None:
                        ctx.param[param.name] == param.default
        return super().invoke(ctx)


class DynamicOption(click.Option):
    class ContextValuesGetter:
        def __init__(self, ctx):
            self._ctx = ctx

        def __getattribute__(self, name):
            if name.startswith("_"):
                return object.__getattribute__(self, name)
            else:
                return object.__getattribute__(self, "_ctx").params.get(name, None)

    def get_default(self, ctx, call=True):
        value = ctx.lookup_default(self.name, call=False)

        if value is None:
            if callable(self.default):
                value = self.default(self.ContextValuesGetter(ctx))
            else:
                value = self.default

        if call and callable(value):
            return value()

        return value


def dynamic_option(*args, **kwargs):
    def decorator(func):
        return click.option(*args, **kwargs, cls=DynamicOption)(func)

    return decorator


class ConfigParam(Param):
    # For compatibility with click>=7.0
    def __init__(self, *args, **kwargs):
        super(ConfigParam, self).__init__(*args, **kwargs)
        self.ctx = None

    def parse(self, text):
        if text:
            text = text.strip()
        if self.type.name == "boolean":
            if not text:
                return None
        return super(ConfigParam, self).parse(text)

    def get_error_hint(self, ctx):
        if self.ctx:
            files = []
            for path in self.ctx.config_searchpath:
                for filename in self.ctx.config_files:
                    files.append(os.path.join(path, filename))
            files = " or ".join(files)
            msg = f"{self.name} in {files}"
        else:
            msg = f"{self.name} in a config file"
        return msg



from hamlet.backend.generate.cmdb import account


conf = dict(
    cmdb=dict(
        tenant=dict(
            tenant_id="tenant_id",
            tenant_name="tenant_name",
            default_region="ap-southeast-2",
            audit_log_expiry_days=2555,
            audit_log_offline_days=90,
        ),
        account=dict(
            account_id="account_id",
            account_name="account_name",
            account_seed=account.generate_account_seed(),
            provider_type="aws",
            provider_id="provider_id",
        ),
        product=dict(
            product_id="product_id",
            product_name="product_name",
            environment_id="environment_id",
            environment_name="environment_name",
            segment_id="segment_id",
            segment_name="segment_name",
            dns_zone="product.local",
        ),
    )
)



import boto3
from base64 import b64encode


def run(
    KeyArn,
    Value,
    EncryptionScheme="",
    Region=None,
    AWSAccessKeyId=None,
    AWSSecretAccessKey=None,
    AWSSessionToken=None,
    env={},
):
    """
    Given a string use aws kms to encrypt the string and return the value
    as a base64 encoded string of the ciphertext blog as the result
    """

    session = boto3.Session(
        aws_access_key_id=AWSAccessKeyId,
        aws_secret_access_key=AWSSecretAccessKey,
        aws_session_token=AWSSessionToken,
        region_name=Region,
    )

    kms = session.client("kms")
    result = kms.encrypt(
        KeyId=KeyArn,
        Plaintext=bytes(Value, "utf-8"),
    )

    return {
        "Properties": {
            "result": f"{EncryptionScheme}{b64encode(result['CiphertextBlob']).decode('utf-8')}"
        }
    }



import os
import json
from marshmallow import ValidationError
from hamlet.backend.common.exceptions import BackendException
from .renderer import testcases_template
from .testcase_schema import Testcase as TestcaseSchema


TESTCASE_EXT = "-testcase.json"


def run(filenames=None, output=None, directory=None):
    filenames = filenames or []
    # searching testcase files in given directory if no files provided
    if not filenames and directory:
        for name in os.listdir(directory):
            if name.endswith(TESTCASE_EXT):
                filenames.append(os.path.join(directory, name))
    # if after all no files found raise an error
    if not filenames:
        raise BackendException("No testcase files found!")
    # merging casefiles
    cases = dict()
    for filename in filenames:
        if not filename.endswith(TESTCASE_EXT):
            raise BackendException(
                f"Invalid extension for {filename}. Must be {TESTCASE_EXT}"
            )
        with open(filename, "rt") as f:
            testcase_file_data = json.load(f)
            for name, testcase in testcase_file_data.items():
                try:
                    TestcaseSchema().load(testcase)
                except ValidationError as e:
                    message = json.dumps(e.messages, indent=4)
                    raise BackendException(
                        f'Invalid testcase schema in {filename}, testcase "{name}". \n\nErrors: \n{message}'
                    ) from e
            cases.update(**testcase_file_data)

    # if files have no testcases data
    if not cases:
        raise BackendException("No testcases found!")

    text = testcases_template.render(cases)
    if output is not None:
        with open(output, "wt") as f:
            f.write(text)
        return ""
    return text



from hamlet.backend.create import template as template_backend


def run(
    StackOutputContent,
    DeploymentUnit,
    DeploymentGroup,
    RootDir,
    Tenant,
    Account,
    Product,
    Environment,
    Segment,
    DistrictType,
    GenerationInputSource,
    GenerationProviders,
    GenerationFramework,
    env={},
):
    """
    Write a stack output file with the provided content
    """

    template_backend.run(
        entrance="stackoutput",
        entrance_parameter=(f"StackOutputContent={StackOutputContent}"),
        deployment_unit=DeploymentUnit,
        deployment_group=DeploymentGroup,
        deployment_mode="update",
        generation_provider=tuple(GenerationProviders.split(",")),
        generation_framework=GenerationFramework,
        generation_input_source=GenerationInputSource,
        district_type=DistrictType,
        root_dir=RootDir,
        tenant=Tenant,
        account=Account,
        environment=Environment,
        product=Product,
        segment=Segment,
        engine=env["_engine"],
        _is_cli=False,
    )

    return {"Properties": {}}



import boto3
from simple_term_menu import TerminalMenu
import json


def run(
    ClusterArn=None,
    TaskFamily=None,
    ServiceName=None,
    Region=None,
    AWSAccessKeyId=None,
    AWSSecretAccessKey=None,
    AWSSessionToken=None,
    env={},
):
    """
    Lists the available tasks for a given service or task definition
    Users select one and the result is returned
    """
    session = boto3.Session(
        aws_access_key_id=AWSAccessKeyId,
        aws_secret_access_key=AWSSecretAccessKey,
        aws_session_token=AWSSessionToken,
        region_name=Region,
    )

    ecs = session.client("ecs")

    taskArns = []

    list_token = None
    while True:

        params = {
            "cluster": ClusterArn,
            "desiredStatus": "RUNNING",
        }

        if TaskFamily is not None:
            params["family"] = TaskFamily

        if ServiceName is not None:
            params["serviceName"] = ServiceName

        if list_token is not None:
            params["nextToken"] = list_token

        list_response = ecs.list_tasks(**params)
        taskArns += list_response["taskArns"]

        if list_response.get("nextToken", None) is not None:
            list_token = list_response["nextToken"]
        else:
            break

    task_details = ecs.describe_tasks(cluster=ClusterArn, tasks=taskArns)

    def get_task_preview(task_arn):

        task = list(
            filter(lambda task: task["taskArn"] == task_arn, task_details["tasks"])
        )[0]

        task_summary = {
            "containers": [
                {k: v for k, v in container.items() if k in ("name", "image")}
                for container in task["containers"]
            ],
            "startedAt": str(task["startedAt"]),
            "startedBy": task["startedBy"],
        }

        return json.dumps(task_summary, indent=2)

    menu = TerminalMenu(
        taskArns,
        title="Tasks",
        preview_command=get_task_preview,
        preview_title="details",
        preview_size=0.75,
    )

    print("\n")
    index = menu.show()
    task_arn = taskArns[index]

    print(f"Task Arn: {task_arn}")
    print("\n")

    return {"Properties": {"result": task_arn}}
